# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import glob
import os
from collections import defaultdict

import datasets


_CITATION = """
@misc{martino2020semeval2020,
      title={SemEval-2020 Task 11: Detection of Propaganda Techniques in News Articles},
      author={G. Da San Martino and A. Barrón-Cedeño and H. Wachsmuth and R. Petrov and P. Nakov},
      year={2020},
      eprint={2009.02696},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
"""

_DESCRIPTION = """\
Propagandistic news articles use specific techniques to convey their message,
such as whataboutism, red Herring, and name calling, among many others.
The Propaganda Techniques Corpus (PTC) allows to study automatic algorithms to
detect them. We provide a permanent leaderboard to allow researchers both to
advertise their progress and to be up-to-speed with the state of the art on the
tasks offered (see below for a definition).
"""

_HOMEPAGE = "https://propaganda.qcri.org/ptc/index.html"

_LICENSE = ""


class SemEval_2020Task_11(datasets.GeneratorBasedBuilder):
    """Semeval 2020 task 11 propaganda detection dataset"""

    VERSION = datasets.Version("1.1.0")

    @property
    def manual_download_instructions(self):
        return """\
        To access the data for this task, register for the task at:
        https://propaganda.qcri.org/ptc/registration.php

        Once registered, go to the main page (https://propaganda.qcri.org/ptc/index.html)
        and enter your passcode to access your team page.

        Alternately, your team page can be access directly with your passcode via the url:
        https://propaganda.qcri.org/ptc/teampage.php?passcode=<YOUR_PASSCODE_HERE>

        From your team page, click on the download link for "PTC Corpus - Version 2".

        Untar this file with `tar -xvf ptc-corpus.tgz`, which will produce a directory called
        `datasets` which contains the subdirectories for text and annotations.

        To load the dataset, pass in the full path to the `datasets` directory
        in your call to `datasets.load_dataset('sem_eval_2020_task_11', data_dir=<path_to_datasets_dir>/datasets)`
        """

    def _info(self):
        features = datasets.Features(
            {
                "article_id": datasets.Value("string"),
                "text": datasets.Value("string"),
                "span_identification": datasets.features.Sequence(
                    {"start_char_offset": datasets.Value("int64"), "end_char_offset": datasets.Value("int64")}
                ),
                "technique_classification": datasets.features.Sequence(
                    {
                        "start_char_offset": datasets.Value("int64"),
                        "end_char_offset": datasets.Value("int64"),
                        "technique": datasets.features.ClassLabel(
                            names=[
                                "Appeal_to_Authority",
                                "Appeal_to_fear-prejudice",
                                "Bandwagon,Reductio_ad_hitlerum",
                                "Black-and-White_Fallacy",
                                "Causal_Oversimplification",
                                "Doubt",
                                "Exaggeration,Minimisation",
                                "Flag-Waving",
                                "Loaded_Language",
                                "Name_Calling,Labeling",
                                "Repetition",
                                "Slogans",
                                "Thought-terminating_Cliches",
                                "Whataboutism,Straw_Men,Red_Herring",
                            ]
                        ),
                    }
                ),
            }
        )

        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # This defines the different columns of the dataset and their types
            features=features,  # Here we define them above because they are different between the two configurations
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_HOMEPAGE,
            # License for the dataset if available
            license=_LICENSE,
            # Citation for the dataset
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        path_to_manual_file = os.path.abspath(os.path.expanduser(dl_manager.manual_dir))

        tc_labels_template = os.path.join(path_to_manual_file, "test-task-tc-template.out")
        tc_test_template = self._process_tc_labels_template(tc_labels_template)

        keys = {}
        for split in ["train", "dev", "test"]:
            articles_path = os.path.join(path_to_manual_file, f"{split}-articles")
            articles_files = glob.glob(os.path.join(articles_path, "*.txt"))
            keys[split] = [os.path.splitext(os.path.basename(af))[0] for af in articles_files]

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"data_dir": path_to_manual_file, "keys": keys["train"], "split": "train", "labels": True},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "data_dir": path_to_manual_file,
                    "keys": keys["test"],
                    "split": "test",
                    "labels": False,
                    "tc_test_template": tc_test_template,
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"data_dir": path_to_manual_file, "keys": keys["dev"], "split": "dev", "labels": True},
            ),
        ]

    def _generate_examples(self, data_dir, keys, split, labels, tc_test_template=None):
        """Yields examples."""

        # Get the main path for the articles
        articles_path = os.path.join(data_dir, f"{split}-articles")

        # If the split has labels, get the main paths for labels
        if labels:
            si_labels_dir = os.path.join(data_dir, f"{split}-labels-task-si")
            tc_labels_dir = os.path.join(data_dir, f"{split}-labels-task-flc-tc")

        # Iterate through each article
        for id_, key in enumerate(sorted(keys)):
            text_path = os.path.join(articles_path, key + ".txt")

            # Read the text for the article
            with open(text_path, encoding="utf-8") as f:
                text = f.read()

            # If the split has labels, load and parse the labels data
            if labels:

                # Get the path for the span labels for the current article
                # and load/split (tab-delimited) label file
                si_labels_path = os.path.join(si_labels_dir, f"{key}.task-si.labels")
                with open(si_labels_path, encoding="utf-8") as f:
                    si_labels = f.readlines()
                si_labels = [label.rstrip("\n").split("\t") for label in si_labels]

                # Span identification task is binary span classification,
                # so there is no associated label for the span
                # (i.e. tagged spans here belong to the positive class)
                span_identification = [
                    {"start_char_offset": int(i[1]), "end_char_offset": int(i[2])} for i in si_labels
                ]

                # Get the path for the technique labels for the current article
                # and load/split (tab-delimited) label file
                tc_labels_path = os.path.join(tc_labels_dir, f"{key}.task-flc-tc.labels")
                with open(tc_labels_path, encoding="utf-8") as f:
                    tc_labels = f.readlines()
                tc_labels = [label.rstrip("\n").split("\t") for label in tc_labels]

                # Technique classification task is a multi-class span classification task
                # so we load the start/end char offsets _as well as_ the class label for the span
                technique_classification = [
                    {"start_char_offset": int(i[2]), "end_char_offset": int(i[3]), "technique": i[1]}
                    for i in tc_labels
                ]

            else:
                # If the split _doesn't_ have labels, return empty lists
                # for the span and technique classification labels
                span_identification = []
                technique_classification = []

                # Add span offsets for technique classification task if provided
                if tc_test_template is not None:
                    tc_labels = tc_test_template[key]
                    technique_classification = [
                        {"start_char_offset": int(i[2]), "end_char_offset": int(i[3]), "technique": -1}
                        for i in tc_labels
                    ]

            yield id_, {
                "article_id": key,
                "text": text,
                "span_identification": span_identification,
                "technique_classification": technique_classification,
            }

    def _process_tc_labels_template(self, tc_labels_template):
        with open(tc_labels_template, encoding="utf-8") as f:
            tc_labels_test = f.readlines()

        tc_labels_test = [line.rstrip("\n").split("\t") for line in tc_labels_test]

        tc_test_template = defaultdict(lambda: [])

        for tc_label in tc_labels_test:
            key = tc_label[0]
            tc_test_template[f"article{key}"].append(tc_label)

        return tc_test_template



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Igbo Monolingual Dataset"""


import json
import os

import datasets


_CITATION = """\
@misc{ezeani2020igboenglish,
title={Igbo-English Machine Translation: An Evaluation Benchmark},
author={Ignatius Ezeani and Paul Rayson and Ikechukwu Onyenwe and Chinedu Uchechukwu and Mark Hepple},
year={2020},
eprint={2004.00648},
archivePrefix={arXiv},
primaryClass={cs.CL}
}
"""

_DESCRIPTION = """\
A dataset is a collection of Monolingual Igbo sentences.
"""

_HOMEPAGE = "https://github.com/IgnatiusEzeani/IGBONLP/tree/master/ig_monoling"

_URL = "https://github.com/IgnatiusEzeani/IGBONLP/raw/master/ig_monoling/json.zip"


class IgboMonolingual(datasets.GeneratorBasedBuilder):
    """collection of Monolingual Igbo sentences from different sources"""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="eze_goes_to_school",
            version=VERSION,
            description="Igbo sentences from Master's Thesis; UZOALOR, JOY UCHE",
        ),
        datasets.BuilderConfig(
            name="bbc-igbo", version=VERSION, description="Igbo sentences from https://www.bbc.com/igbo/"
        ),
        datasets.BuilderConfig(
            name="igbo-radio", version=VERSION, description="Igbo sentences from https://www.igboradio.com"
        ),
        datasets.BuilderConfig(
            name="jw-ot-igbo", version=VERSION, description="Igbo sentences from 929 pages of 39 books"
        ),
        datasets.BuilderConfig(
            name="jw-nt-igbo", version=VERSION, description="Igbo sentences from 260 pages of 27 books"
        ),
        datasets.BuilderConfig(
            name="jw-books", version=VERSION, description="Igbo sentences from 2204 pages of 48 books "
        ),
        datasets.BuilderConfig(
            name="jw-teta", version=VERSION, description="Igbo sentences from 392 pages of 37 magazines"
        ),
        datasets.BuilderConfig(
            name="jw-ulo_nche", version=VERSION, description="Igbo sentences from 594 pages of 55 magazines"
        ),
        datasets.BuilderConfig(
            name="jw-ulo_nche_naamu", version=VERSION, description="Igbo sentences from 870 pages of 88 magazines"
        ),
    ]

    def _info(self):
        if self.config.name == "eze_goes_to_school":
            features = datasets.Features(
                {
                    "format": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "chapters": datasets.Sequence(
                        {"title": datasets.Value("string"), "content": datasets.Value("string")}
                    ),
                }
            )
        elif self.config.name == "bbc-igbo":
            features = datasets.Features(
                {
                    "source": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "description": datasets.Value("string"),
                    "date": datasets.Value("string"),
                    "headline": datasets.Value("string"),
                    "content": datasets.Value("string"),
                    "tags": datasets.Sequence(datasets.Value("string")),
                }
            )
        elif self.config.name == "igbo-radio":
            features = datasets.Features(
                {
                    "source": datasets.Value("string"),
                    "headline": datasets.Value("string"),
                    "author": datasets.Value("string"),
                    "date": datasets.Value("string"),
                    "description": datasets.Value("string"),
                    "content": datasets.Value("string"),
                }
            )
        elif self.config.name == "jw-ot-igbo":
            features = datasets.Features(
                {
                    "format": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "chapters": datasets.Sequence(
                        {"title": datasets.Value("string"), "content": datasets.Value("string")}
                    ),
                }
            )
        elif self.config.name == "jw-nt-igbo":
            features = datasets.Features(
                {
                    "format": datasets.Value("string"),
                    "title": datasets.Value("string"),
                    "chapters": datasets.Sequence(
                        {"title": datasets.Value("string"), "content": datasets.Value("string")}
                    ),
                }
            )
        elif self.config.name == "jw-books":
            features = datasets.Features(
                {
                    "title": datasets.Value("string"),
                    "content": datasets.Value("string"),
                    "format": datasets.Value("string"),
                    "date": datasets.Value("string"),
                }
            )
        elif self.config.name == "jw-teta":
            features = datasets.Features(
                {
                    "title": datasets.Value("string"),
                    "content": datasets.Value("string"),
                    "format": datasets.Value("string"),
                    "date": datasets.Value("string"),
                }
            )
        elif self.config.name == "jw-ulo_nche":
            features = datasets.Features(
                {
                    "title": datasets.Value("string"),
                    "content": datasets.Value("string"),
                    "format": datasets.Value("string"),
                    "date": datasets.Value("string"),
                }
            )
        elif self.config.name == "jw-ulo_nche_naamu":
            features = datasets.Features(
                {
                    "title": datasets.Value("string"),
                    "content": datasets.Value("string"),
                    "format": datasets.Value("string"),
                    "date": datasets.Value("string"),
                }
            )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_dir = dl_manager.download_and_extract(_URL)
        if self.config.name == "eze_goes_to_school":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "eze_goes_to_school.json"),
                        "split": "train",
                    },
                ),
            ]
        elif self.config.name == "bbc-igbo":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "bbc-igbo.json"),
                        "split": "train",
                    },
                ),
            ]
        elif self.config.name == "igbo-radio":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "igbo-radio.json"),
                        "split": "train",
                    },
                ),
            ]
        elif self.config.name == "jw-ot-igbo":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "jw-ot-igbo.json"),
                        "split": "train",
                    },
                ),
            ]
        elif self.config.name == "jw-nt-igbo":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "jw-nt-igbo.json"),
                        "split": "train",
                    },
                ),
            ]
        elif self.config.name == "jw-books":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "jw-books.json"),
                        "split": "train",
                    },
                ),
            ]
        elif self.config.name == "jw-teta":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "jw-teta.json"),
                        "split": "train",
                    },
                ),
            ]
        elif self.config.name == "jw-ulo_nche":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "jw-ulo_nche.json"),
                        "split": "train",
                    },
                ),
            ]
        elif self.config.name == "jw-ulo_nche_naamu":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "jw-ulo_nche_naamu.json"),
                        "split": "train",
                    },
                ),
            ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        if self.config.name == "eze_goes_to_school":
            with open(filepath, "r", encoding="utf-8") as f:
                json_data = json.load(f)
                for id_, data in enumerate(json_data):
                    yield id_, {
                        "format": data["format"],
                        "title": data["title"],
                        "chapters": data["chapters"],
                    }
        elif self.config.name == "bbc-igbo":
            with open(filepath, "r", encoding="utf-8") as f:
                json_data = json.load(f)
                for id_, data in enumerate(json_data):
                    yield id_, {
                        "source": data["source"] if "source" in data.keys() else "",
                        "title": data["title"] if "title" in data.keys() else "",
                        "description": data["description"] if "description" in data.keys() else "",
                        "date": data["date"] if "date" in data.keys() else "",
                        "headline": data["headline"] if "headline" in data.keys() else "",
                        "content": data["content"] if "content" in data.keys() else "",
                        "tags": data["tags"] if "tags" in data.keys() else [],
                    }
        elif self.config.name == "igbo-radio":
            with open(filepath, "r", encoding="utf-8") as f:
                json_data = json.load(f)
                for id_, data in enumerate(json_data):
                    yield id_, {
                        "source": data["source"],
                        "headline": data["headline"],
                        "author": data["author"],
                        "date": data["date"],
                        "description": data["description"] if "description" in data.keys() else "",
                        "content": data["content"] if "content" in data.keys() else "",
                    }
        elif self.config.name == "jw-ot-igbo":
            with open(filepath, "r", encoding="utf-8") as f:
                json_data = json.load(f)
                for id_, data in enumerate(json_data):
                    yield id_, {
                        "format": data["format"],
                        "title": data["title"],
                        "chapters": data["chapters"],
                    }
        elif self.config.name == "jw-nt-igbo":
            with open(filepath, "r", encoding="utf-8") as f:
                json_data = json.load(f)
                for id_, data in enumerate(json_data):
                    yield id_, {
                        "format": data["format"],
                        "title": data["title"],
                        "chapters": data["chapters"],
                    }
        elif self.config.name == "jw-books":
            with open(filepath, "r", encoding="utf-8") as f:
                json_data = json.load(f)
                for id_, data in enumerate(json_data):
                    yield id_, {
                        "title": data["title"],
                        "content": data["content"],
                        "format": data["format"],
                        "date": data["date"] if "date" in data.keys() else "",
                    }
        elif self.config.name == "jw-teta":
            with open(filepath, "r", encoding="utf-8") as f:
                json_data = json.load(f)
                for id_, data in enumerate(json_data):
                    yield id_, {
                        "title": data["title"],
                        "content": data["content"],
                        "format": data["format"],
                        "date": data["date"] if "date" in data.keys() else "",
                    }
        elif self.config.name == "jw-ulo_nche":
            with open(filepath, "r", encoding="utf-8") as f:
                json_data = json.load(f)
                for id_, data in enumerate(json_data):
                    yield id_, {
                        "title": data["title"],
                        "content": data["content"],
                        "format": data["format"],
                        "date": data["date"] if "date" in data.keys() else "",
                    }
        elif self.config.name == "jw-ulo_nche_naamu":
            with open(filepath, "r", encoding="utf-8") as f:
                json_data = json.load(f)
                for id_, data in enumerate(json_data):
                    yield id_, {
                        "title": data["title"],
                        "content": data["content"],
                        "format": data["format"],
                        "date": data["date"] if "date" in data.keys() else "",
                    }



"""Scientific fact-checking dataset. Verifies claims based on citation sentences
using evidence from the cited abstracts."""


import json

import datasets


_CITATION = """\
@inproceedings{Wadden2020FactOF,
  title={Fact or Fiction: Verifying Scientific Claims},
  author={David Wadden and Shanchuan Lin and Kyle Lo and Lucy Lu Wang and Madeleine van Zuylen and Arman Cohan and Hannaneh Hajishirzi},
  booktitle={EMNLP},
  year={2020},
}
"""

_DESCRIPTION = """\
SciFact, a dataset of 1.4K expert-written scientific claims paired with evidence-containing abstracts, and annotated with labels and rationales.
"""

_URL = "https://scifact.s3-us-west-2.amazonaws.com/release/latest/data.tar.gz"


class ScifactConfig(datasets.BuilderConfig):
    """BuilderConfig for Scifact"""

    def __init__(self, **kwargs):
        """

        Args:
            **kwargs: keyword arguments forwarded to super.
        """
        super(ScifactConfig, self).__init__(version=datasets.Version("1.0.0", ""), **kwargs)


class Scifact(datasets.GeneratorBasedBuilder):
    """TODO(scifact): Short description of my dataset."""

    # TODO(scifact): Set up version.
    VERSION = datasets.Version("0.1.0")
    BUILDER_CONFIGS = [
        ScifactConfig(name="corpus", description=" The corpus of evidence documents"),
        ScifactConfig(name="claims", description=" The claims are split into train, test, dev"),
    ]

    def _info(self):
        # TODO(scifact): Specifies the datasets.DatasetInfo object
        if self.config.name == "corpus":
            features = {
                "doc_id": datasets.Value("int32"),  # The document's S2ORC ID.
                "title": datasets.Value("string"),  # The title.
                "abstract": datasets.features.Sequence(
                    datasets.Value("string")
                ),  # The abstract, written as a list of sentences.
                "structured": datasets.Value("bool"),  # Indicator for whether this is a structured abstract.
            }
        else:
            features = {
                "id": datasets.Value("int32"),  # An integer claim ID.
                "claim": datasets.Value("string"),  # The text of the claim.
                "evidence_doc_id": datasets.Value("string"),
                "evidence_label": datasets.Value("string"),  # Label for the rationale.
                "evidence_sentences": datasets.features.Sequence(datasets.Value("int32")),  # Rationale sentences.
                "cited_doc_ids": datasets.features.Sequence(datasets.Value("int32")),  # The claim's "cited documents".
            }

        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                features
                # These are the features of your dataset like images, labels ...
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="https://scifact.apps.allenai.org/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(scifact): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        archive = dl_manager.download(_URL)

        if self.config.name == "corpus":
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": "data/corpus.jsonl",
                        "split": "train",
                        "files": dl_manager.iter_archive(archive),
                    },
                ),
            ]
        else:
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": "data/claims_train.jsonl",
                        "split": "train",
                        "files": dl_manager.iter_archive(archive),
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": "data/claims_test.jsonl",
                        "split": "test",
                        "files": dl_manager.iter_archive(archive),
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": "data/claims_dev.jsonl",
                        "split": "dev",
                        "files": dl_manager.iter_archive(archive),
                    },
                ),
            ]

    def _generate_examples(self, filepath, split, files):
        """Yields examples."""
        # TODO(scifact): Yields (key, example) tuples from the dataset
        for path, f in files:
            if path == filepath:
                for id_, row in enumerate(f):
                    data = json.loads(row.decode("utf-8"))
                    if self.config.name == "corpus":
                        yield id_, {
                            "doc_id": int(data["doc_id"]),
                            "title": data["title"],
                            "abstract": data["abstract"],
                            "structured": data["structured"],
                        }
                    else:
                        if split == "test":
                            yield id_, {
                                "id": data["id"],
                                "claim": data["claim"],
                                "evidence_doc_id": "",
                                "evidence_label": "",
                                "evidence_sentences": [],
                                "cited_doc_ids": [],
                            }
                        else:
                            evidences = data["evidence"]
                            if evidences:
                                for id1, doc_id in enumerate(evidences):
                                    for id2, evidence in enumerate(evidences[doc_id]):
                                        yield str(id_) + "_" + str(id1) + "_" + str(id2), {
                                            "id": data["id"],
                                            "claim": data["claim"],
                                            "evidence_doc_id": doc_id,
                                            "evidence_label": evidence["label"],
                                            "evidence_sentences": evidence["sentences"],
                                            "cited_doc_ids": data.get("cited_doc_ids", []),
                                        }
                            else:
                                yield id_, {
                                    "id": data["id"],
                                    "claim": data["claim"],
                                    "evidence_doc_id": "",
                                    "evidence_label": "",
                                    "evidence_sentences": [],
                                    "cited_doc_ids": data.get("cited_doc_ids", []),
                                }
                break



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Nergrit Corpus"""


import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """\
@inproceedings{id_nergrit_corpus,
  author    = {Gria Inovasi Teknologi},
  title     = {NERGRIT CORPUS},
  year      = {2019},
  url       = {https://github.com/grit-id/nergrit-corpus},
}
"""

_DESCRIPTION = """\
Nergrit Corpus is a dataset collection for Indonesian Named Entity Recognition, Statement Extraction, and Sentiment
Analysis. id_nergrit_corpus is the Named Entity Recognition of this dataset collection which contains 18 entities as
follow:
    'CRD': Cardinal
    'DAT': Date
    'EVT': Event
    'FAC': Facility
    'GPE': Geopolitical Entity
    'LAW': Law Entity (such as Undang-Undang)
    'LOC': Location
    'MON': Money
    'NOR': Political Organization
    'ORD': Ordinal
    'ORG': Organization
    'PER': Person
    'PRC': Percent
    'PRD': Product
    'QTY': Quantity
    'REG': Religion
    'TIM': Time
    'WOA': Work of Art
    'LAN': Language
"""

_HOMEPAGE = "https://github.com/grit-id/nergrit-corpus"

_LICENSE = ""

_URLs = [
    "https://github.com/cahya-wirawan/indonesian-language-models/raw/master/data/nergrit-corpus_20190726_corrected.tgz",
    "https://cloud.uncool.ai/index.php/s/2QEcMrgwkjMAo4o/download",
]


class IdNergritCorpusConfig(datasets.BuilderConfig):
    """BuilderConfig for IdNergritCorpus"""

    def __init__(self, label_classes=None, **kwargs):
        """BuilderConfig for IdNergritCorpus.
        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(IdNergritCorpusConfig, self).__init__(**kwargs)
        self.label_classes = label_classes


class IdNergritCorpus(datasets.GeneratorBasedBuilder):
    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        IdNergritCorpusConfig(
            name="ner",
            version=VERSION,
            description="Named Entity Recognition dataset of Nergrit Corpus",
            label_classes=[
                "B-CRD",
                "B-DAT",
                "B-EVT",
                "B-FAC",
                "B-GPE",
                "B-LAN",
                "B-LAW",
                "B-LOC",
                "B-MON",
                "B-NOR",
                "B-ORD",
                "B-ORG",
                "B-PER",
                "B-PRC",
                "B-PRD",
                "B-QTY",
                "B-REG",
                "B-TIM",
                "B-WOA",
                "I-CRD",
                "I-DAT",
                "I-EVT",
                "I-FAC",
                "I-GPE",
                "I-LAN",
                "I-LAW",
                "I-LOC",
                "I-MON",
                "I-NOR",
                "I-ORD",
                "I-ORG",
                "I-PER",
                "I-PRC",
                "I-PRD",
                "I-QTY",
                "I-REG",
                "I-TIM",
                "I-WOA",
                "O",
            ],
        ),
        IdNergritCorpusConfig(
            name="sentiment",
            version=VERSION,
            description="Sentiment Analysis dataset of Nergrit Corpus",
            label_classes=[
                "B-NEG",
                "B-NET",
                "B-POS",
                "I-NEG",
                "I-NET",
                "I-POS",
                "O",
            ],
        ),
        IdNergritCorpusConfig(
            name="statement",
            version=VERSION,
            description="Statement Extraction dataset of Nergrit Corpus",
            label_classes=[
                "B-BREL",
                "B-FREL",
                "B-STAT",
                "B-WHO",
                "I-BREL",
                "I-FREL",
                "I-STAT",
                "I-WHO",
                "O",
            ],
        ),
    ]

    def _info(self):
        features = datasets.Features(
            {
                "id": datasets.Value("string"),
                "tokens": datasets.Sequence(datasets.Value("string")),
                "ner_tags": datasets.Sequence(datasets.features.ClassLabel(names=self.config.label_classes)),
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        my_urls = _URLs[0]
        archive = dl_manager.download(my_urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": f"nergrit-corpus/{self.config.name}/data/train_corrected.txt",
                    "split": "train",
                    "files": dl_manager.iter_archive(archive),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filepath": f"nergrit-corpus/{self.config.name}/data/test_corrected.txt",
                    "split": "test",
                    "files": dl_manager.iter_archive(archive),
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": f"nergrit-corpus/{self.config.name}/data/valid_corrected.txt",
                    "split": "dev",
                    "files": dl_manager.iter_archive(archive),
                },
            ),
        ]

    def _generate_examples(self, filepath, split, files):
        for path, f in files:
            if path == filepath:
                guid = 0
                tokens = []
                ner_tags = []
                for line in f:
                    splits = line.decode("utf-8").strip().split()
                    if len(splits) != 2:
                        if tokens:
                            assert len(tokens) == len(ner_tags), "word len doesn't match label length"
                            yield guid, {
                                "id": str(guid),
                                "tokens": tokens,
                                "ner_tags": ner_tags,
                            }
                            guid += 1
                            tokens = []
                            ner_tags = []
                    else:
                        tokens.append(splits[0])
                        ner_tags.append(splits[1].rstrip())
                # last example
                yield guid, {
                    "id": str(guid),
                    "tokens": tokens,
                    "ner_tags": ner_tags,
                }
                break



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""AQuaMuSe is a novel scalable approach to automatically mine dual query based multi-document summarization datasets for extractive and abstractive summaries using question answering dataset (Google Natural Questions) and large document corpora (Common Crawl)"""


import os
from os import listdir
from os.path import isfile, join

import tensorflow as tf

import datasets


_CITATION = """\
@misc{kulkarni2020aquamuse,
      title={AQuaMuSe: Automatically Generating Datasets for Query-Based Multi-Document Summarization},
      author={Sayali Kulkarni and Sheide Chammas and Wan Zhu and Fei Sha and Eugene Ie},
      year={2020},
      eprint={2010.12694},
      archivePrefix={arXiv},
      primaryClass={cs.CL}
}
"""

_DESCRIPTION = """AQuaMuSe is a novel scalable approach to automatically mine dual query based multi-document summarization datasets for extractive and abstractive summaries using question answering dataset (Google Natural Questions) and large document corpora (Common Crawl)"""

_HOMEPAGE = "https://github.com/google-research-datasets/aquamuse"

_LICENSE = ""

zipped_data_url = "https://github.com/google-research-datasets/aquamuse/raw/main/v2/aquamuse_v2.zip"


class Aquamuse(datasets.GeneratorBasedBuilder):
    """Dataset for Query-based Multi-Document Summarization"""

    VERSION = datasets.Version("2.3.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="abstractive", version=VERSION, description="Abstractive query-based multi-document summarization"
        ),
        datasets.BuilderConfig(
            name="extractive", version=VERSION, description="Extractive query-based multi-document summarization"
        ),
    ]

    # DEFAULT_CONFIG_NAME = "abstractive"  # It's not mandatory to have a default configuration. Just use one if it make sense.

    def _info(self):
        features = datasets.Features(
            {
                "query": datasets.Value("string"),
                "input_urls": datasets.Sequence(datasets.Value("string")),
                "target": datasets.Value("string"),
            }
        )

        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        if self.config.name == "abstractive":
            data_dir = dl_manager.download_and_extract(zipped_data_url)
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "v2.3/abstractive/train/"),
                        "split": "train",
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "v2.3/abstractive/test/"),
                        "split": "test",
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "v2.3/abstractive/dev/"),
                        "split": "dev",
                    },
                ),
            ]

        else:
            data_dir = dl_manager.download_and_extract(zipped_data_url)
            return [
                datasets.SplitGenerator(
                    name=datasets.Split.TRAIN,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "v2.3/extractive/train/"),
                        "split": "train",
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "v2.3/extractive/test/"),
                        "split": "test",
                    },
                ),
                datasets.SplitGenerator(
                    name=datasets.Split.VALIDATION,
                    # These kwargs will be passed to _generate_examples
                    gen_kwargs={
                        "filepath": os.path.join(data_dir, "v2.3/extractive/dev/"),
                        "split": "dev",
                    },
                ),
            ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        filepath = [join(filepath, f) for f in listdir(filepath) if isfile(join(filepath, f))]
        filepath = sorted(filepath)
        raw_dataset = tf.data.TFRecordDataset(filepath)
        for id_, raw_record in enumerate(raw_dataset):
            example = tf.train.Example()
            example.ParseFromString(raw_record.numpy())
            yield id_, {
                "query": example.features.feature["query"].bytes_list.value[0].decode(),
                "input_urls": example.features.feature["input_urls"].bytes_list.value[0].decode().split("<EOD>"),
                "target": example.features.feature["target"].bytes_list.value[0].decode(),
            }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Fake News Filipino Dataset"""

import csv
import os

import datasets
from datasets.tasks import TextClassification


_DESCRIPTION = """\
    Low-Resource Fake News Detection Corpora in Filipino. The first of its kind. Contains 3,206 expertly-labeled news samples, half of which are real and half of which are fake.
"""

_CITATION = """\
    @inproceedings{cruz2020localization,
      title={Localization of Fake News Detection via Multitask Transfer Learning},
      author={Cruz, Jan Christian Blaise and Tan, Julianne Agatha and Cheng, Charibeth},
      booktitle={Proceedings of The 12th Language Resources and Evaluation Conference},
      pages={2596--2604},
      year={2020}
    }
"""

_HOMEPAGE = "https://github.com/jcblaisecruz02/Tagalog-fake-news"

# TODO: Add the licence for the dataset here if you can find it
_LICENSE = ""

_URL = "https://s3.us-east-2.amazonaws.com/blaisecruz.com/datasets/fakenews/fakenews.zip"


class FakeNewsFilipino(datasets.GeneratorBasedBuilder):
    """Low-Resource Fake News Detection Corpora in Filipino"""

    VERSION = datasets.Version("1.0.0")

    def _info(self):
        features = datasets.Features(
            {"label": datasets.features.ClassLabel(names=["0", "1"]), "article": datasets.Value("string")}
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            license=_LICENSE,
            citation=_CITATION,
            task_templates=[TextClassification(text_column="article", label_column="label")],
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_dir = dl_manager.download_and_extract(_URL)
        train_path = os.path.join(data_dir, "fakenews", "full.csv")

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": train_path,
                    "split": "train",
                },
            )
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        with open(filepath, encoding="utf-8") as csv_file:
            csv_reader = csv.reader(
                csv_file, quotechar='"', delimiter=",", quoting=csv.QUOTE_ALL, skipinitialspace=True
            )
            next(csv_reader)
            for id_, row in enumerate(csv_reader):
                label, article = row
                yield id_, {"label": label, "article": article}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The WikiANN dataset for multilingual named entity recognition"""


import os

import datasets


_CITATION = """@inproceedings{pan-etal-2017-cross,
    title = "Cross-lingual Name Tagging and Linking for 282 Languages",
    author = "Pan, Xiaoman  and
      Zhang, Boliang  and
      May, Jonathan  and
      Nothman, Joel  and
      Knight, Kevin  and
      Ji, Heng",
    booktitle = "Proceedings of the 55th Annual Meeting of the Association for Computational Linguistics (Volume 1: Long Papers)",
    month = jul,
    year = "2017",
    address = "Vancouver, Canada",
    publisher = "Association for Computational Linguistics",
    url = "https://www.aclweb.org/anthology/P17-1178",
    doi = "10.18653/v1/P17-1178",
    pages = "1946--1958",
    abstract = "The ambitious goal of this work is to develop a cross-lingual name tagging and linking framework for 282 languages that exist in Wikipedia. Given a document in any of these languages, our framework is able to identify name mentions, assign a coarse-grained or fine-grained type to each mention, and link it to an English Knowledge Base (KB) if it is linkable. We achieve this goal by performing a series of new KB mining methods: generating {``}silver-standard{''} annotations by transferring annotations from English to other languages through cross-lingual links and KB properties, refining annotations through self-training and topic selection, deriving language-specific morphology features from anchor links, and mining word translation pairs from cross-lingual links. Both name tagging and linking results for 282 languages are promising on Wikipedia data and on-Wikipedia data.",
}"""

_DESCRIPTION = """WikiANN (sometimes called PAN-X) is a multilingual named entity recognition dataset consisting of Wikipedia articles annotated with LOC (location), PER (person), and ORG (organisation) tags in the IOB2 format. This version corresponds to the balanced train, dev, and test splits of Rahimi et al. (2019), which supports 176 of the 282 languages from the original WikiANN corpus."""

_DATA_URL = "https://s3.amazonaws.com/datasets.huggingface.co/wikiann/1.1.0/panx_dataset.zip"
_HOMEPAGE = "https://github.com/afshinrahimi/mmner"
_VERSION = "1.1.0"
_LANGS = [
    "ace",
    "af",
    "als",
    "am",
    "an",
    "ang",
    "ar",
    "arc",
    "arz",
    "as",
    "ast",
    "ay",
    "az",
    "ba",
    "bar",
    "bat-smg",
    "be",
    "be-x-old",
    "bg",
    "bh",
    "bn",
    "bo",
    "br",
    "bs",
    "ca",
    "cbk-zam",
    "cdo",
    "ce",
    "ceb",
    "ckb",
    "co",
    "crh",
    "cs",
    "csb",
    "cv",
    "cy",
    "da",
    "de",
    "diq",
    "dv",
    "el",
    "eml",
    "en",
    "eo",
    "es",
    "et",
    "eu",
    "ext",
    "fa",
    "fi",
    "fiu-vro",
    "fo",
    "fr",
    "frr",
    "fur",
    "fy",
    "ga",
    "gan",
    "gd",
    "gl",
    "gn",
    "gu",
    "hak",
    "he",
    "hi",
    "hr",
    "hsb",
    "hu",
    "hy",
    "ia",
    "id",
    "ig",
    "ilo",
    "io",
    "is",
    "it",
    "ja",
    "jbo",
    "jv",
    "ka",
    "kk",
    "km",
    "kn",
    "ko",
    "ksh",
    "ku",
    "ky",
    "la",
    "lb",
    "li",
    "lij",
    "lmo",
    "ln",
    "lt",
    "lv",
    "map-bms",
    "mg",
    "mhr",
    "mi",
    "min",
    "mk",
    "ml",
    "mn",
    "mr",
    "ms",
    "mt",
    "mwl",
    "my",
    "mzn",
    "nap",
    "nds",
    "ne",
    "nl",
    "nn",
    "no",
    "nov",
    "oc",
    "or",
    "os",
    "pa",
    "pdc",
    "pl",
    "pms",
    "pnb",
    "ps",
    "pt",
    "qu",
    "rm",
    "ro",
    "ru",
    "rw",
    "sa",
    "sah",
    "scn",
    "sco",
    "sd",
    "sh",
    "si",
    "simple",
    "sk",
    "sl",
    "so",
    "sq",
    "sr",
    "su",
    "sv",
    "sw",
    "szl",
    "ta",
    "te",
    "tg",
    "th",
    "tk",
    "tl",
    "tr",
    "tt",
    "ug",
    "uk",
    "ur",
    "uz",
    "vec",
    "vep",
    "vi",
    "vls",
    "vo",
    "wa",
    "war",
    "wuu",
    "xmf",
    "yi",
    "yo",
    "zea",
    "zh",
    "zh-classical",
    "zh-min-nan",
    "zh-yue",
]


class WikiannConfig(datasets.BuilderConfig):
    def __init__(self, **kwargs):
        super(WikiannConfig, self).__init__(version=datasets.Version(_VERSION, ""), **kwargs)


class Wikiann(datasets.GeneratorBasedBuilder):
    """WikiANN is a multilingual named entity recognition dataset consisting of Wikipedia articles annotated with LOC, PER, and ORG tags"""

    VERSION = datasets.Version(_VERSION)
    # use two-letter ISO 639-1 language codes as the name for each corpus
    BUILDER_CONFIGS = [
        WikiannConfig(name=lang, description=f"WikiANN NER examples in language {lang}") for lang in _LANGS
    ]

    def _tags_to_spans(self, tags):
        """Convert tags to spans."""
        spans = set()
        span_start = 0
        span_end = 0
        active_conll_tag = None
        for index, string_tag in enumerate(tags):
            # Actual BIO tag.
            bio_tag = string_tag[0]
            assert bio_tag in ["B", "I", "O"], "Invalid Tag"
            conll_tag = string_tag[2:]
            if bio_tag == "O":
                # The span has ended.
                if active_conll_tag:
                    spans.add((active_conll_tag, (span_start, span_end)))
                active_conll_tag = None
                # We don't care about tags we are
                # told to ignore, so we do nothing.
                continue
            elif bio_tag == "B":
                # We are entering a new span; reset indices and active tag to new span.
                if active_conll_tag:
                    spans.add((active_conll_tag, (span_start, span_end)))
                active_conll_tag = conll_tag
                span_start = index
                span_end = index
            elif bio_tag == "I" and conll_tag == active_conll_tag:
                # We're inside a span.
                span_end += 1
            else:
                # This is the case the bio label is an "I", but either:
                # 1) the span hasn't started - i.e. an ill formed span.
                # 2) We have IOB1 tagging scheme.
                # We'll process the previous span if it exists, but also include this
                # span. This is important, because otherwise, a model may get a perfect
                # F1 score whilst still including false positive ill-formed spans.
                if active_conll_tag:
                    spans.add((active_conll_tag, (span_start, span_end)))
                active_conll_tag = conll_tag
                span_start = index
                span_end = index
        # Last token might have been a part of a valid span.
        if active_conll_tag:
            spans.add((active_conll_tag, (span_start, span_end)))
        # Return sorted list of spans
        return sorted(list(spans), key=lambda x: x[1][0])

    def _get_spans(self, tokens, tags):
        """Convert tags to textspans."""
        spans = self._tags_to_spans(tags)
        text_spans = [x[0] + ": " + " ".join([tokens[i] for i in range(x[1][0], x[1][1] + 1)]) for x in spans]
        if not text_spans:
            text_spans = ["None"]
        return text_spans

    def _info(self):
        features = datasets.Features(
            {
                "tokens": datasets.Sequence(datasets.Value("string")),
                "ner_tags": datasets.Sequence(
                    datasets.features.ClassLabel(
                        names=[
                            "O",
                            "B-PER",
                            "I-PER",
                            "B-ORG",
                            "I-ORG",
                            "B-LOC",
                            "I-LOC",
                        ]
                    )
                ),
                "langs": datasets.Sequence(datasets.Value("string")),
                "spans": datasets.Sequence(datasets.Value("string")),
            }
        )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        wikiann_dl_dir = dl_manager.download_and_extract(_DATA_URL)
        lang = self.config.name
        lang_archive = os.path.join(wikiann_dl_dir, lang + ".tar.gz")

        return [
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": "dev", "files": dl_manager.iter_archive(lang_archive)},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": "test", "files": dl_manager.iter_archive(lang_archive)},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": "train", "files": dl_manager.iter_archive(lang_archive)},
            ),
        ]

    def _generate_examples(self, filepath, files):
        """Reads line by line format of the NER dataset and generates examples.
        Input Format:
        en:rick  B-PER
        en:and  O
        en:morty  B-PER
        en:are  O
        en:cool  O
        en:.  O
        Output Format:
        {
        'tokens': ["rick", "and", "morty", "are", "cool", "."],
        'ner_tags': ["B-PER", "O" , "B-PER", "O", "O", "O"],
        'langs': ["en", "en", "en", "en", "en", "en"]
        'spans': ["PER: rick", "PER: morty"]
        }
        Args:
            filepath: Path to file with line by line NER format.
        Returns:
            Examples with the format listed above.
        """
        guid_index = 1
        for path, f in files:
            if path == filepath:
                tokens = []
                ner_tags = []
                langs = []
                for line in f:
                    line = line.decode("utf-8")
                    if line == "" or line == "\n":
                        if tokens:
                            spans = self._get_spans(tokens, ner_tags)
                            yield guid_index, {"tokens": tokens, "ner_tags": ner_tags, "langs": langs, "spans": spans}
                            guid_index += 1
                            tokens = []
                            ner_tags = []
                            langs = []
                    else:
                        # wikiann data is tab separated
                        splits = line.split("\t")
                        # strip out en: prefix
                        langs.append(splits[0].split(":")[0])
                        tokens.append(":".join(splits[0].split(":")[1:]))
                        if len(splits) > 1:
                            ner_tags.append(splits[-1].replace("\n", ""))
                        else:
                            # examples have no label in test set
                            ner_tags.append("O")
                break



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Conceptnet 5.7.0 and OMCSNet raw data"""


import json

import datasets


# Find for instance the citation on arxiv or on the dataset repo/website
_CITATION = """\
Robyn Speer, Joshua Chin, and Catherine Havasi. 2017. "ConceptNet 5.5: An Open Multilingual Graph of General Knowledge." In proceedings of AAAI 31.
}
"""

# You can copy an official description
_DESCRIPTION = """This dataset is designed to provide training data
for common sense relationships pulls together from various sources.

The dataset is multi-lingual. See langauge codes and language info
here: https://github.com/commonsense/conceptnet5/wiki/Languages


This dataset provides an interface for the conceptnet5 csv file, and
some (but not all) of the raw text data used to build conceptnet5:
omcsnet_sentences_free.txt, and omcsnet_sentences_more.txt.

One use of this dataset would be to learn to extract the conceptnet
relationship from the omcsnet sentences.

Conceptnet5 has 34,074,917 relationships. Of those relationships,
there are 2,176,099 surface text sentences related to those 2M
entries.

omcsnet_sentences_free has 898,161 lines. omcsnet_sentences_more has
2,001,736 lines.

Original downloads are available here
https://github.com/commonsense/conceptnet5/wiki/Downloads. For more
information, see: https://github.com/commonsense/conceptnet5/wiki

The omcsnet data comes with the following warning from the authors of
the above site: Remember: this data comes from various forms of
crowdsourcing. Sentences in these files are not necessarily true,
useful, or appropriate.

"""


_LICENSE = """
This work includes data from ConceptNet 5, which was compiled by the
Commonsense Computing Initiative. ConceptNet 5 is freely available under
the Creative Commons Attribution-ShareAlike license (CC BY SA 3.0) from
http://conceptnet.io.

The included data was created by contributors to Commonsense Computing
projects, contributors to Wikimedia projects, DBPedia, OpenCyc, Games
with a Purpose, Princeton University's WordNet, Francis Bond's Open
Multilingual WordNet, and Jim Breen's JMDict.

There are various othe licenses. See:
https://github.com/commonsense/conceptnet5/wiki/Copying-and-sharing-ConceptNet
"""

_URLs = {
    "conceptnet5": "https://s3.amazonaws.com/conceptnet/downloads/2019/edges/conceptnet-assertions-5.7.0.csv.gz",
    "omcs_sentences_free": "https://s3.amazonaws.com/conceptnet/downloads/2018/omcs-sentences-free.txt",
    "omcs_sentences_more": "https://s3.amazonaws.com/conceptnet/downloads/2018/omcs-sentences-more.txt",
}


class Conceptnet5(datasets.GeneratorBasedBuilder):
    """Conceptnet5 dataset for common sense graphs and underlying sentences."""

    VERSION = datasets.Version("0.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="conceptnet5", description="The relationships defined by conceptnet5", version="5.7.0"
        ),
        datasets.BuilderConfig(name="omcs_sentences_free", description="OMCSNet free form text", version="5.7.0"),
        datasets.BuilderConfig(
            name="omcs_sentences_more",
            description="OMCSNet free form text, and text from templates, games, responses to questions, and so on",
            version="5.7.0",
        ),
    ]

    DEFAULT_CONFIG_NAME = "conceptnet5"

    def _info(self):
        if self.config.name == "conceptnet5":
            features = datasets.Features(
                {
                    "sentence": datasets.Value("string"),
                    "full_rel": datasets.Value("string"),
                    "rel": datasets.Value("string"),
                    "arg1": datasets.Value("string"),
                    "arg2": datasets.Value("string"),
                    "lang": datasets.Value("string"),
                    "extra_info": datasets.Value("string"),
                    "weight": datasets.Value("float"),
                }
            )
        else:
            features = datasets.Features(
                {
                    "sentence": datasets.Value("string"),
                    "raw_data": datasets.Value("string"),
                    "lang": datasets.Value("string"),
                }
            )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage="https://github.com/commonsense/conceptnet5/wiki",
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        my_urls = _URLs[self.config.name]
        data_dir = dl_manager.download_and_extract(my_urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples from the conceptnet5 graph if the config is 'conceptnet5', otherwise yields the sentences for omcs."""

        with open(filepath, "rb") as f:
            for id_, row in enumerate(f):
                if self.config.name == "conceptnet5":
                    row = row.split(b"\t")
                    s = row[4]
                    data = json.loads(s)
                    lang1 = row[2].split(b"/")[2].decode("utf-8")
                    lang2 = row[3].split(b"/")[2].decode("utf-8")
                    if lang1 == lang2:
                        lang = lang1
                    else:
                        lang = lang1 + "/" + lang2
                    if "surfaceText" in data:
                        sentence = data["surfaceText"].strip()
                    else:
                        sentence = ""
                    if b"weight" in data:
                        weight = float(data[b"weight"])
                    else:
                        weight = 1.0
                    yield id_, {
                        "sentence": sentence,
                        "full_rel": row[0].decode("utf-8"),
                        "rel": row[1].decode("utf-8"),
                        "arg1": row[2].decode("utf-8"),
                        "arg2": row[3].decode("utf-8"),
                        "lang": lang,
                        "extra_info": row[4].decode("utf-8"),
                        "weight": weight,
                    }
                else:
                    row = row.decode("utf-8").strip()
                    data = row.split("\t")
                    if len(data) > 1:
                        sentence = data[1]
                        lang = data[4]
                    else:
                        continue
                    yield id_, {
                        "raw_data": row,
                        "sentence": sentence,
                        "lang": lang,
                    }



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""The NumerSense Dataset"""


import csv

import datasets


_CITATION = """\
@inproceedings{lin2020numersense,
    title={Birds have four legs?! NumerSense: Probing Numerical Commonsense Knowledge of Pre-trained Language Models},
    author={Bill Yuchen Lin and Seyeon Lee and Rahul Khanna and Xiang Ren},
    booktitle={Proceedings of EMNLP},
    year={2020},
    note={to appear}
}
"""

_DESCRIPTION = """\
NumerSense is a new numerical commonsense reasoning probing task, with a diagnostic dataset consisting of 3,145 masked-word-prediction probes.

We propose to study whether numerical commonsense knowledge can be induced from pre-trained language models like BERT, and to what extent this access to knowledge robust against adversarial examples is. We hope this will be beneficial for tasks such as knowledge base completion and open-domain question answering.
"""

_HOMEPAGE_URL = "https://inklab.usc.edu/NumerSense/"
_BASE_DOWNLOAD_URL = "https://raw.githubusercontent.com/INK-USC/NumerSense/main/data/"


class NumerSense(datasets.GeneratorBasedBuilder):
    """The Multilingual Amazon Reviews Corpus"""

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "sentence": datasets.Value("string"),
                    "target": datasets.Value("string"),
                }
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE_URL,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        train_url = _BASE_DOWNLOAD_URL + "train.masked.tsv"
        test_core_url = _BASE_DOWNLOAD_URL + "test.core.masked.txt"
        test_all_url = _BASE_DOWNLOAD_URL + "test.all.masked.txt"

        train_path = dl_manager.download_and_extract(train_url)
        test_core_path = dl_manager.download_and_extract(test_core_url)
        test_all_path = dl_manager.download_and_extract(test_all_url)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"file_path": train_path, "is_test": False}),
            datasets.SplitGenerator(name="test_core", gen_kwargs={"file_path": test_core_path, "is_test": True}),
            datasets.SplitGenerator(name="test_all", gen_kwargs={"file_path": test_all_path, "is_test": True}),
        ]

    def _generate_examples(self, file_path, is_test):
        with open(file_path, "r", encoding="utf-8") as f:
            if is_test:
                for i, sentence in enumerate(f):
                    yield i, {"sentence": sentence.rstrip(), "target": ""}
            else:
                reader = csv.DictReader(f, delimiter="\t", fieldnames=["sentence", "target"])
                for i, row in enumerate(reader):
                    yield i, row



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
import os

import datasets


_DESCRIPTION = """\
This is a collection of copyright free books aligned by Andras Farkas, which are available from http://www.farkastranslations.com/bilingual_books.php
Note that the texts are rather dated due to copyright issues and that some of them are manually reviewed (check the meta-data at the top of the corpus files in XML). The source is multilingually aligned, which is available from http://www.farkastranslations.com/bilingual_books.php. In OPUS, the alignment is formally bilingual but the multilingual alignment can be recovered from the XCES sentence alignment files. Note also that the alignment units from the original source may include multi-sentence paragraphs, which are split and sentence-aligned in OPUS.
All texts are freely available for personal, educational and research use. Commercial use (e.g. reselling as parallel books) and mass redistribution without explicit permission are not granted. Please acknowledge the source when using the data!

16 languages, 64 bitexts
total number of files: 158
total number of tokens: 19.50M
total number of sentence fragments: 0.91M
"""
_HOMEPAGE_URL = "http://opus.nlpl.eu/Books.php"
_CITATION = """\
@InProceedings{TIEDEMANN12.463,
  author = {J�rg Tiedemann},
  title = {Parallel Data, Tools and Interfaces in OPUS},
  booktitle = {Proceedings of the Eight International Conference on Language Resources and Evaluation (LREC'12)},
  year = {2012},
  month = {may},
  date = {23-25},
  address = {Istanbul, Turkey},
  editor = {Nicoletta Calzolari (Conference Chair) and Khalid Choukri and Thierry Declerck and Mehmet Ugur Dogan and Bente Maegaard and Joseph Mariani and Jan Odijk and Stelios Piperidis},
  publisher = {European Language Resources Association (ELRA)},
  isbn = {978-2-9517408-7-7},
  language = {english}
 }
"""

_VERSION = "1.0.0"
_BASE_NAME = "Books.{}.{}"
_BASE_URL = "https://object.pouta.csc.fi/OPUS-Books/v1/moses/{}-{}.txt.zip"

_LANGUAGE_PAIRS = [
    ("ca", "de"),
    ("ca", "en"),
    ("de", "en"),
    ("el", "en"),
    ("de", "eo"),
    ("en", "eo"),
    ("de", "es"),
    ("el", "es"),
    ("en", "es"),
    ("eo", "es"),
    ("en", "fi"),
    ("es", "fi"),
    ("de", "fr"),
    ("el", "fr"),
    ("en", "fr"),
    ("eo", "fr"),
    ("es", "fr"),
    ("fi", "fr"),
    ("ca", "hu"),
    ("de", "hu"),
    ("el", "hu"),
    ("en", "hu"),
    ("eo", "hu"),
    ("fr", "hu"),
    ("de", "it"),
    ("en", "it"),
    ("eo", "it"),
    ("es", "it"),
    ("fr", "it"),
    ("hu", "it"),
    ("ca", "nl"),
    ("de", "nl"),
    ("en", "nl"),
    ("es", "nl"),
    ("fr", "nl"),
    ("hu", "nl"),
    ("it", "nl"),
    ("en", "no"),
    ("es", "no"),
    ("fi", "no"),
    ("fr", "no"),
    ("hu", "no"),
    ("en", "pl"),
    ("fi", "pl"),
    ("fr", "pl"),
    ("hu", "pl"),
    ("de", "pt"),
    ("en", "pt"),
    ("eo", "pt"),
    ("es", "pt"),
    ("fr", "pt"),
    ("hu", "pt"),
    ("it", "pt"),
    ("de", "ru"),
    ("en", "ru"),
    ("es", "ru"),
    ("fr", "ru"),
    ("hu", "ru"),
    ("it", "ru"),
    ("en", "sv"),
    ("fr", "sv"),
    ("it", "sv"),
]


class OpusBooksConfig(datasets.BuilderConfig):
    def __init__(self, *args, lang1=None, lang2=None, **kwargs):
        super().__init__(
            *args,
            name=f"{lang1}-{lang2}",
            **kwargs,
        )
        self.lang1 = lang1
        self.lang2 = lang2


class OpusBooks(datasets.GeneratorBasedBuilder):
    BUILDER_CONFIGS = [
        OpusBooksConfig(
            lang1=lang1,
            lang2=lang2,
            description=f"Translating {lang1} to {lang2} or vice versa",
            version=datasets.Version(_VERSION),
        )
        for lang1, lang2 in _LANGUAGE_PAIRS
    ]
    BUILDER_CONFIG_CLASS = OpusBooksConfig

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "translation": datasets.Translation(languages=(self.config.lang1, self.config.lang2)),
                },
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE_URL,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        def _base_url(lang1, lang2):
            return _BASE_URL.format(lang1, lang2)

        download_url = _base_url(self.config.lang1, self.config.lang2)
        path = dl_manager.download_and_extract(download_url)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"datapath": path},
            )
        ]

    def _generate_examples(self, datapath):
        l1, l2 = self.config.lang1, self.config.lang2
        folder = l1 + "-" + l2
        l1_file = _BASE_NAME.format(folder, l1)
        l2_file = _BASE_NAME.format(folder, l2)
        l1_path = os.path.join(datapath, l1_file)
        l2_path = os.path.join(datapath, l2_file)
        with open(l1_path, encoding="utf-8") as f1, open(l2_path, encoding="utf-8") as f2:
            for sentence_counter, (x, y) in enumerate(zip(f1, f2)):
                x = x.strip()
                y = y.strip()
                result = (
                    sentence_counter,
                    {
                        "id": str(sentence_counter),
                        "translation": {l1: x, l2: y},
                    },
                )
                yield result



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Dataset of 10065 tweets in Arabic for Emotion detection in Arabic text """


import csv

import datasets
from datasets.tasks import TextClassification


_CITATION = """\
@inbook{inbook,
author = {Al-Khatib, Amr and El-Beltagy, Samhaa},
year = {2018},
month = {01},
pages = {105-114},
title = {Emotional Tone Detection in Arabic Tweets: 18th International Conference, CICLing 2017, Budapest, Hungary, April 17–23, 2017, Revised Selected Papers, Part II},
isbn = {978-3-319-77115-1},
doi = {10.1007/978-3-319-77116-8_8}
}
"""


_DESCRIPTION = """\
Dataset of 10065 tweets in Arabic for Emotion detection in Arabic text"""


_HOMEPAGE = "https://github.com/AmrMehasseb/Emotional-Tone"


_DOWNLOAD_URL = "https://raw.githubusercontent.com/AmrMehasseb/Emotional-Tone/master/Emotional-Tone-Dataset.csv"


class EmotoneAr(datasets.GeneratorBasedBuilder):
    """Dataset of 10065 tweets in Arabic for Emotions detection in Arabic text"""

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "tweet": datasets.Value("string"),
                    "label": datasets.features.ClassLabel(
                        names=["none", "anger", "joy", "sadness", "love", "sympathy", "surprise", "fear"]
                    ),
                }
            ),
            homepage=_HOMEPAGE,
            citation=_CITATION,
            task_templates=[TextClassification(text_column="tweet", label_column="label")],
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        data_dir = dl_manager.download_and_extract(_DOWNLOAD_URL)
        return [datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": data_dir})]

    def _generate_examples(self, filepath):
        """Generate labeled arabic tweets examples for emoptions detection."""
        with open(filepath, encoding="utf-8", mode="r") as csv_file:
            next(csv_file)  # skip header
            csv_reader = csv.reader(csv_file, quotechar='"', delimiter=",")

            for id_, row in enumerate(csv_reader):
                _, tweet, label = row
                yield id_, {"tweet": tweet, "label": label}



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""CivilComments from Jigsaw Unintended Bias Kaggle Competition."""


import csv
import os

import datasets


_CITATION = """
@article{DBLP:journals/corr/abs-1903-04561,
  author    = {Daniel Borkan and
               Lucas Dixon and
               Jeffrey Sorensen and
               Nithum Thain and
               Lucy Vasserman},
  title     = {Nuanced Metrics for Measuring Unintended Bias with Real Data for Text
               Classification},
  journal   = {CoRR},
  volume    = {abs/1903.04561},
  year      = {2019},
  url       = {http://arxiv.org/abs/1903.04561},
  archivePrefix = {arXiv},
  eprint    = {1903.04561},
  timestamp = {Sun, 31 Mar 2019 19:01:24 +0200},
  biburl    = {https://dblp.org/rec/bib/journals/corr/abs-1903-04561},
  bibsource = {dblp computer science bibliography, https://dblp.org}
}
"""

_DESCRIPTION = """
The comments in this dataset come from an archive of the Civil Comments
platform, a commenting plugin for independent news sites. These public comments
were created from 2015 - 2017 and appeared on approximately 50 English-language
news sites across the world. When Civil Comments shut down in 2017, they chose
to make the public comments available in a lasting open archive to enable future
research. The original data, published on figshare, includes the public comment
text, some associated metadata such as article IDs, timestamps and
commenter-generated "civility" labels, but does not include user ids. Jigsaw
extended this dataset by adding additional labels for toxicity and identity
mentions. This data set is an exact replica of the data released for the
Jigsaw Unintended Bias in Toxicity Classification Kaggle challenge.  This
dataset is released under CC0, as is the underlying comment text.
"""

_DOWNLOAD_URL = "https://storage.googleapis.com/jigsaw-unintended-bias-in-toxicity-classification/civil_comments.zip"


class CivilComments(datasets.GeneratorBasedBuilder):
    """Classification and tagging of 2M comments on news sites.

    This version of the CivilComments Dataset provides access to the primary
    seven labels that were annotated by crowd workers, the toxicity and other
    tags are a value between 0 and 1 indicating the fraction of annotators that
    assigned these attributes to the comment text.

    The other tags, which are only available for a fraction of the input examples
    are currently ignored, as are all of the attributes that were part of the
    original civil comments release. See the Kaggle documentation for more
    details about the available features.
    """

    VERSION = datasets.Version("0.9.0")

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "toxicity": datasets.Value("float32"),
                    "severe_toxicity": datasets.Value("float32"),
                    "obscene": datasets.Value("float32"),
                    "threat": datasets.Value("float32"),
                    "insult": datasets.Value("float32"),
                    "identity_attack": datasets.Value("float32"),
                    "sexual_explicit": datasets.Value("float32"),
                }
            ),
            # The supervised_keys version is very impoverished.
            supervised_keys=("text", "toxicity"),
            homepage="https://www.kaggle.com/c/jigsaw-unintended-bias-in-toxicity-classification/data",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        dl_path = dl_manager.download_and_extract(_DOWNLOAD_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filename": os.path.join(dl_path, "train.csv"), "toxicity_label": "target"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filename": os.path.join(dl_path, "test_public_expanded.csv"),
                    "toxicity_label": "toxicity",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={
                    "filename": os.path.join(dl_path, "test_private_expanded.csv"),
                    "toxicity_label": "toxicity",
                },
            ),
        ]

    def _generate_examples(self, filename, toxicity_label):
        """Yields examples.

        Each example contains a text input and then seven annotation labels.

        Args:
          filename: the path of the file to be read for this split.
          toxicity_label: indicates 'target' or 'toxicity' to capture the variation
            in the released labels for this dataset.

        Yields:
          A dictionary of features, all floating point except the input text.
        """
        with open(filename, encoding="utf-8") as f:
            reader = csv.DictReader(f)
            for row in reader:
                example = {}
                example["text"] = row["comment_text"]
                example["toxicity"] = float(row[toxicity_label])
                for label in ["severe_toxicity", "obscene", "threat", "insult", "identity_attack", "sexual_explicit"]:
                    example[label] = float(row[label])
                yield row["id"], example



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Igbo Named Entity Recognition Dataset"""


import datasets


_CITATION = """\
@misc{ezeani2020igboenglish,
    title={Igbo-English Machine Translation: An Evaluation Benchmark},
    author={Ignatius Ezeani and Paul Rayson and Ikechukwu Onyenwe and Chinedu Uchechukwu and Mark Hepple},
    year={2020},
    eprint={2004.00648},
    archivePrefix={arXiv},
    primaryClass={cs.CL}
}
"""

_DESCRIPTION = """\
Igbo Named Entity Recognition Dataset
"""

_HOMEPAGE = "https://github.com/IgnatiusEzeani/IGBONLP/tree/master/ig_ner"

_URLs = {
    "ner_data": "https://raw.githubusercontent.com/IgnatiusEzeani/IGBONLP/master/ig_ner/igbo_data.txt",
    "free_text": "https://raw.githubusercontent.com/IgnatiusEzeani/IGBONLP/master/ig_ner/igbo_data10000.txt",
}


class IgboNer(datasets.GeneratorBasedBuilder):
    """Dataset from the Igbo NER Project"""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(
            name="ner_data",
            version=VERSION,
            description="This dataset contains the named entity and all the sentences containing that entity.",
        ),
        datasets.BuilderConfig(
            name="free_text", version=VERSION, description="This dataset contains all sentences used for NER."
        ),
    ]

    DEFAULT_CONFIG_NAME = "ner_data"

    def _info(self):
        if self.config.name == "ner_data":
            features = datasets.Features(
                {
                    "content_n": datasets.Value("string"),
                    "named_entity": datasets.Value("string"),
                    "sentences": datasets.Sequence(datasets.Value("string")),
                }
            )
        else:
            features = datasets.Features(
                {
                    "sentences": datasets.Value("string"),
                }
            )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage=_HOMEPAGE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        my_urls = _URLs[self.config.name]
        data_dir = dl_manager.download_and_extract(my_urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir,
                    "split": "train",
                },
            ),
        ]

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        dictionary = {}
        with open(filepath, "r", encoding="utf-8-sig") as f:
            if self.config.name == "ner_data":
                for id_, row in enumerate(f):
                    row = row.strip().split("\t")
                    content_n = row[0]
                    if content_n in dictionary.keys():
                        (dictionary[content_n]["sentences"]).append(row[1])
                    else:
                        dictionary[content_n] = {}
                        dictionary[content_n]["named_entity"] = row[1]
                        dictionary[content_n]["sentences"] = [row[1]]
                    yield id_, {
                        "content_n": content_n,
                        "named_entity": dictionary[content_n]["named_entity"],
                        "sentences": dictionary[content_n]["sentences"],
                    }
            else:
                for id_, row in enumerate(f):
                    yield id_, {
                        "sentences": row.strip(),
                    }



"""TODO(qangaroo): Add a description here."""


import json
import os

import datasets


# TODO(qangaroo): BibTeX citation

_CITATION = """
"""

# TODO(quangaroo):
_DESCRIPTION = """\
  We have created two new Reading Comprehension datasets focussing on multi-hop (alias multi-step) inference.

Several pieces of information often jointly imply another fact. In multi-hop inference, a new fact is derived by combining facts via a chain of multiple steps.

Our aim is to build Reading Comprehension methods that perform multi-hop inference on text, where individual facts are spread out across different documents.

The two QAngaroo datasets provide a training and evaluation resource for such methods.
"""

_MEDHOP_DESCRIPTION = """\
  With the same format as WikiHop, this dataset is based on research paper abstracts from PubMed, and the queries are about interactions between pairs of drugs.
  The correct answer has to be inferred by combining information from a chain of reactions of drugs and proteins.
  """
_WIKIHOP_DESCRIPTION = """\
  With the same format as WikiHop, this dataset is based on research paper abstracts from PubMed, and the queries are about interactions between pairs of drugs.
   The correct answer has to be inferred by combining information from a chain of reactions of drugs and proteins.
  """

_URL = "https://drive.google.com/uc?export=download&id=1ytVZ4AhubFDOEL7o7XrIRIyhU8g9wvKA"


class QangarooConfig(datasets.BuilderConfig):
    def __init__(self, data_dir, **kwargs):
        """BuilderConfig for qangaroo dataset

        Args:
          data_dir: directory for the given dataset name
          **kwargs: keyword arguments forwarded to super.

        """

        super(QangarooConfig, self).__init__(version=datasets.Version("1.0.0", ""), **kwargs)

        self.data_dir = data_dir


class Qangaroo(datasets.GeneratorBasedBuilder):
    """TODO(qangaroo): Short description of my dataset."""

    # TODO(qangaroo): Set up version.
    VERSION = datasets.Version("0.1.0")
    BUILDER_CONFIGS = [
        QangarooConfig(name="medhop", description=_MEDHOP_DESCRIPTION, data_dir="medhop"),
        QangarooConfig(name="masked_medhop", description=_MEDHOP_DESCRIPTION, data_dir="medhop"),
        QangarooConfig(name="wikihop", description=_WIKIHOP_DESCRIPTION, data_dir="wikihop"),
        QangarooConfig(name="masked_wikihop", description=_WIKIHOP_DESCRIPTION, data_dir="wikihop"),
    ]

    def _info(self):
        # TODO(qangaroo): Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # datasets.features.FeatureConnectors
            features=datasets.Features(
                {
                    # These are the features of your dataset like images, labels ...
                    "query": datasets.Value("string"),
                    "supports": datasets.features.Sequence(datasets.Value("string")),
                    "candidates": datasets.features.Sequence(datasets.Value("string")),
                    "answer": datasets.Value("string"),
                    "id": datasets.Value("string")
                    # These are the features of your dataset like images, labels ...
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage="http://qangaroo.cs.ucl.ac.uk/index.html",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # TODO(qangaroo): Downloads the data and defines the splits
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        dl_dir = dl_manager.download_and_extract(_URL)
        data_dir = os.path.join(dl_dir, "qangaroo_v1.1")
        train_file = "train.masked.json" if "masked" in self.config.name else "train.json"
        dev_file = "dev.masked.json" if "masked" in self.config.name else "dev.json"
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": os.path.join(data_dir, self.config.data_dir, train_file)},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"filepath": os.path.join(data_dir, self.config.data_dir, dev_file)},
            ),
        ]

    def _generate_examples(self, filepath):
        """Yields examples."""
        # TODO(quangaroo): Yields (key, example) tuples from the dataset
        with open(filepath, encoding="utf-8") as f:
            data = json.load(f)
            for example in data:
                id_ = example["id"]
                yield id_, {
                    "id": example["id"],
                    "query": example["query"],
                    "supports": example["supports"],
                    "candidates": example["candidates"],
                    "answer": example["answer"],
                }



import glob
import os
from pathlib import Path

import datasets


_CITATION = """\
@article{boonkwan2020annotation,
  title={The Annotation Guideline of LST20 Corpus},
  author={Boonkwan, Prachya and Luantangsrisuk, Vorapon and Phaholphinyo, Sitthaa and Kriengket, Kanyanat and Leenoi, Dhanon and Phrombut, Charun and Boriboon, Monthika and Kosawat, Krit and Supnithi, Thepchai},
  journal={arXiv preprint arXiv:2008.05055},
  year={2020}
}
"""

_DESCRIPTION = """\
LST20 Corpus is a dataset for Thai language processing developed by National Electronics and Computer Technology Center (NECTEC), Thailand.
It offers five layers of linguistic annotation: word boundaries, POS tagging, named entities, clause boundaries, and sentence boundaries.
At a large scale, it consists of 3,164,002 words, 288,020 named entities, 248,181 clauses, and 74,180 sentences, while it is annotated with
16 distinct POS tags. All 3,745 documents are also annotated with one of 15 news genres. Regarding its sheer size, this dataset is
considered large enough for developing joint neural models for NLP.
Manually download at https://aiforthai.in.th/corpus.php
"""


class Lst20Config(datasets.BuilderConfig):
    """BuilderConfig for Lst20"""

    def __init__(self, **kwargs):
        """BuilderConfig for Lst20.

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(Lst20Config, self).__init__(**kwargs)


class Lst20(datasets.GeneratorBasedBuilder):
    """Lst20 dataset."""

    _SENTENCE_SPLITTERS = ["", " ", "\n"]
    _TRAINING_FOLDER = "train"
    _VALID_FOLDER = "eval"
    _TEST_FOLDER = "test"
    _POS_TAGS = ["NN", "VV", "PU", "CC", "PS", "AX", "AV", "FX", "NU", "AJ", "CL", "PR", "NG", "PA", "XX", "IJ"]
    _NER_TAGS = [
        "O",
        "B_BRN",
        "B_DES",
        "B_DTM",
        "B_LOC",
        "B_MEA",
        "B_NUM",
        "B_ORG",
        "B_PER",
        "B_TRM",
        "B_TTL",
        "I_BRN",
        "I_DES",
        "I_DTM",
        "I_LOC",
        "I_MEA",
        "I_NUM",
        "I_ORG",
        "I_PER",
        "I_TRM",
        "I_TTL",
        "E_BRN",
        "E_DES",
        "E_DTM",
        "E_LOC",
        "E_MEA",
        "E_NUM",
        "E_ORG",
        "E_PER",
        "E_TRM",
        "E_TTL",
    ]
    _CLAUSE_TAGS = ["O", "B_CLS", "I_CLS", "E_CLS"]

    BUILDER_CONFIGS = [
        Lst20Config(name="lst20", version=datasets.Version("1.0.0"), description="LST20 dataset"),
    ]

    @property
    def manual_download_instructions(self):
        return """\
  You need to
  1. Manually download `AIFORTHAI-LST20Corpus.tar.gz` from https://aiforthai.in.th/corpus.php (login required; website mostly in Thai)
  2. Extract the .tar.gz; this will result in folder `LST20Corpus`
  The <path/to/folder> can e.g. be `~/Downloads/LST20Corpus`.
  lst20 can then be loaded using the following command `datasets.load_dataset("lst20", data_dir="<path/to/folder>")`.
  """

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "fname": datasets.Value("string"),
                    "tokens": datasets.Sequence(datasets.Value("string")),
                    "pos_tags": datasets.Sequence(datasets.features.ClassLabel(names=self._POS_TAGS)),
                    "ner_tags": datasets.Sequence(datasets.features.ClassLabel(names=self._NER_TAGS)),
                    "clause_tags": datasets.Sequence(datasets.features.ClassLabel(names=self._CLAUSE_TAGS)),
                }
            ),
            supervised_keys=None,
            homepage="https://aiforthai.in.th/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        data_dir = os.path.abspath(os.path.expanduser(dl_manager.manual_dir))

        # check if manual folder exists
        if not os.path.exists(data_dir):
            raise FileNotFoundError(
                f"{data_dir} does not exist. Make sure you insert a manual dir via `datasetts.load_dataset('lst20', data_dir=...)`. Manual download instructions: {self.manual_download_instructions})"
            )

        # check number of .txt files
        nb_train = len(glob.glob(os.path.join(data_dir, "train", "*.txt")))
        nb_valid = len(glob.glob(os.path.join(data_dir, "eval", "*.txt")))
        nb_test = len(glob.glob(os.path.join(data_dir, "test", "*.txt")))
        assert (
            nb_train > 0
        ), f"No files found in train/*.txt.\nManual download instructions:{self.manual_download_instructions})"
        assert (
            nb_valid > 0
        ), f"No files found in eval/*.txt.\nManual download instructions:{self.manual_download_instructions})"
        assert (
            nb_test > 0
        ), f"No files found in test/*.txt.\nManual download instructions:{self.manual_download_instructions})"

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"filepath": os.path.join(data_dir, self._TRAINING_FOLDER)},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={"filepath": os.path.join(data_dir, self._VALID_FOLDER)},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                gen_kwargs={"filepath": os.path.join(data_dir, self._TEST_FOLDER)},
            ),
        ]

    def _generate_examples(self, filepath):
        for file_idx, fname in enumerate(sorted(glob.glob(os.path.join(filepath, "*.txt")))):
            with open(fname, encoding="utf-8") as f:
                guid = 0
                tokens = []
                pos_tags = []
                ner_tags = []
                clause_tags = []

                for line in f:
                    if line in self._SENTENCE_SPLITTERS:
                        if tokens:
                            yield f"{file_idx}_{guid}", {
                                "id": str(guid),
                                "fname": Path(fname).name,
                                "tokens": tokens,
                                "pos_tags": pos_tags,
                                "ner_tags": ner_tags,
                                "clause_tags": clause_tags,
                            }
                            guid += 1
                            tokens = []
                            pos_tags = []
                            ner_tags = []
                            clause_tags = []
                    else:
                        # LST20 tokens are tab separated
                        splits = line.split("\t")
                        # replace junk ner tags
                        ner_tag = splits[2] if splits[2] in self._NER_TAGS else "O"
                        tokens.append(splits[0])
                        pos_tags.append(splits[1])
                        ner_tags.append(ner_tag)
                        clause_tags.append(splits[3].rstrip())
                # last example
                if tokens:
                    yield f"{file_idx}_{guid}", {
                        "id": str(guid),
                        "fname": Path(fname).name,
                        "tokens": tokens,
                        "pos_tags": pos_tags,
                        "ner_tags": ner_tags,
                        "clause_tags": clause_tags,
                    }



import gzip


class IOHandler:
    def __init__(self, file_name, file_mode, compression):
        self._file_name = file_name
        self._file_mode = file_mode
        self._compression = compression

    def __enter__(self):
        if self._compression is None:
            self._file = open(self._file_name, self._file_mode)
        elif self._compression == "gzip":
            self._file = gzip.open(self._file_name, self._file_mode)

        return self._file

    def __exit__(self, exc_type, exc_value, exc_traceback):
        self._file.close()



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This is an authorship attribution dataset based on the work of Stamatatos 2013. """


import os

import datasets


_CITATION = """\
@article{article,
    author = {Stamatatos, Efstathios},
    year = {2013},
    month = {01},
    pages = {421-439},
    title = {On the robustness of authorship attribution based on character n-gram features},
    volume = {21},
    journal = {Journal of Law and Policy}
}

@inproceedings{stamatatos2017authorship,
    title={Authorship attribution using text distortion},
    author={Stamatatos, Efstathios},
    booktitle={Proc. of the 15th Conf. of the European Chapter of the Association for Computational Linguistics},
    volume={1}
    pages={1138--1149},
    year={2017}
}
"""

_DESCRIPTION = """\
A dataset cross-topic authorship attribution. The dataset is provided by Stamatatos 2013.
1- The cross-topic scenarios are based on Table-4 in Stamatatos 2017 (Ex. cross_topic_1 => row 1:P S U&W ).
2- The cross-genre scenarios are based on Table-5 in the same paper. (Ex. cross_genre_1 => row 1:B P S&U&W).

3- The same-topic/genre scenario is created by grouping all the datasts as follows.
For ex., to use same_topic and split the data 60-40 use:
train_ds = load_dataset('guardian_authorship', name="cross_topic_<<#>>",
                        split='train[:60%]+validation[:60%]+test[:60%]')
tests_ds = load_dataset('guardian_authorship', name="cross_topic_<<#>>",
                        split='train[-40%:]+validation[-40%:]+test[-40%:]')

IMPORTANT: train+validation+test[:60%] will generate the wrong splits becasue the data is imbalanced

* See https://huggingface.co/docs/datasets/splits.html for detailed/more examples
"""

_URL = "https://www.dropbox.com/s/lc5mje0owl9shms/Guardian.zip?dl=1"


# Using a specific configuration class is optional, you can also use the base class if you don't need
# to add specific attributes.
# here we give an example for three sub-set of the dataset with difference sizes.
class GuardianAuthorshipConfig(datasets.BuilderConfig):
    """BuilderConfig for NewDataset"""

    def __init__(self, train_folder, valid_folder, test_folder, **kwargs):
        """
        Args:
            Train_folder: Topic/genre used for training
            valid_folder:       ~      ~   for validation
            test_folder:        ~      ~   for testing

            **kwargs: keyword arguments forwarded to super.
        """
        super(GuardianAuthorshipConfig, self).__init__(**kwargs)
        self.train_folder = train_folder
        self.valid_folder = valid_folder
        self.test_folder = test_folder


class GuardianAuthorship(datasets.GeneratorBasedBuilder):
    """dataset for same- and cross-topic authorship attribution"""

    config_counter = 0
    BUILDER_CONFIG_CLASS = GuardianAuthorshipConfig
    BUILDER_CONFIGS = [
        # cross-topic
        GuardianAuthorshipConfig(
            name=f"cross_topic_{1}",
            version=datasets.Version(f"{1}.0.0", description=f"The Original DS with the cross-topic scenario no.{1}"),
            train_folder="Politics",
            valid_folder="Society",
            test_folder="UK,World",
        ),
        GuardianAuthorshipConfig(
            name=f"cross_topic_{2}",
            version=datasets.Version(f"{2}.0.0", description=f"The Original DS with the cross-topic scenario no.{2}"),
            train_folder="Politics",
            valid_folder="UK",
            test_folder="Society,World",
        ),
        GuardianAuthorshipConfig(
            name=f"cross_topic_{3}",
            version=datasets.Version(f"{3}.0.0", description=f"The Original DS with the cross-topic scenario no.{3}"),
            train_folder="Politics",
            valid_folder="World",
            test_folder="Society,UK",
        ),
        GuardianAuthorshipConfig(
            name=f"cross_topic_{4}",
            version=datasets.Version(f"{4}.0.0", description=f"The Original DS with the cross-topic scenario no.{4}"),
            train_folder="Society",
            valid_folder="Politics",
            test_folder="UK,World",
        ),
        GuardianAuthorshipConfig(
            name=f"cross_topic_{5}",
            version=datasets.Version(f"{5}.0.0", description=f"The Original DS with the cross-topic scenario no.{5}"),
            train_folder="Society",
            valid_folder="UK",
            test_folder="Politics,World",
        ),
        GuardianAuthorshipConfig(
            name=f"cross_topic_{6}",
            version=datasets.Version(f"{6}.0.0", description=f"The Original DS with the cross-topic scenario no.{6}"),
            train_folder="Society",
            valid_folder="World",
            test_folder="Politics,UK",
        ),
        GuardianAuthorshipConfig(
            name=f"cross_topic_{7}",
            version=datasets.Version(f"{7}.0.0", description=f"The Original DS with the cross-topic scenario no.{7}"),
            train_folder="UK",
            valid_folder="Politics",
            test_folder="Society,World",
        ),
        GuardianAuthorshipConfig(
            name=f"cross_topic_{8}",
            version=datasets.Version(f"{8}.0.0", description=f"The Original DS with the cross-topic scenario no.{8}"),
            train_folder="UK",
            valid_folder="Society",
            test_folder="Politics,World",
        ),
        GuardianAuthorshipConfig(
            name=f"cross_topic_{9}",
            version=datasets.Version(f"{9}.0.0", description=f"The Original DS with the cross-topic scenario no.{9}"),
            train_folder="UK",
            valid_folder="World",
            test_folder="Politics,Society",
        ),
        GuardianAuthorshipConfig(
            name=f"cross_topic_{10}",
            version=datasets.Version(
                f"{10}.0.0", description=f"The Original DS with the cross-topic scenario no.{10}"
            ),
            train_folder="World",
            valid_folder="Politics",
            test_folder="Society,UK",
        ),
        GuardianAuthorshipConfig(
            name=f"cross_topic_{11}",
            version=datasets.Version(
                f"{11}.0.0", description=f"The Original DS with the cross-topic scenario no.{11}"
            ),
            train_folder="World",
            valid_folder="Society",
            test_folder="Politics,UK",
        ),
        GuardianAuthorshipConfig(
            name=f"cross_topic_{12}",
            version=datasets.Version(
                f"{12}.0.0", description=f"The Original DS with the cross-topic scenario no.{12}"
            ),
            train_folder="World",
            valid_folder="UK",
            test_folder="Politics,Society",
        ),
        # # cross-genre
        GuardianAuthorshipConfig(
            name=f"cross_genre_{1}",
            version=datasets.Version(f"{1}.0.0", description=f"The Original DS with the cross-genre scenario no.{1}"),
            train_folder="Books",
            valid_folder="Politics",
            test_folder="Society,UK,World",
        ),
        GuardianAuthorshipConfig(
            name=f"cross_genre_{2}",
            version=datasets.Version(f"{2}.0.0", description=f"The Original DS with the cross-genre scenario no.{2}"),
            train_folder="Books",
            valid_folder="Society",
            test_folder="Politics,UK,World",
        ),
        GuardianAuthorshipConfig(
            name=f"cross_genre_{3}",
            version=datasets.Version(f"{3}.0.0", description=f"The Original DS with the cross-genre scenario no.{3}"),
            train_folder="Books",
            valid_folder="UK",
            test_folder="Politics,Society,World",
        ),
        GuardianAuthorshipConfig(
            name=f"cross_genre_{4}",
            version=datasets.Version(f"{4}.0.0", description=f"The Original DS with the cross-genre scenario no.{4}"),
            train_folder="Books",
            valid_folder="World",
            test_folder="Politics,Society,UK",
        ),
    ]

    def _info(self):
        # Specifies the datasets.DatasetInfo object
        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    # These are the features of your dataset like images, labels ...
                    # There are 13 authors in this dataset
                    "author": datasets.features.ClassLabel(
                        names=[
                            "catherinebennett",
                            "georgemonbiot",
                            "hugoyoung",
                            "jonathanfreedland",
                            "martinkettle",
                            "maryriddell",
                            "nickcohen",
                            "peterpreston",
                            "pollytoynbee",
                            "royhattersley",
                            "simonhoggart",
                            "willhutton",
                            "zoewilliams",
                        ]
                    ),
                    # There are book reviews, and articles on the following four topics
                    "topic": datasets.features.ClassLabel(names=["Politics", "Society", "UK", "World", "Books"]),
                    "article": datasets.Value("string"),
                }
            ),
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=[("article", "author")],
            # Homepage of the dataset for documentation
            homepage="http://www.icsd.aegean.gr/lecturers/stamatatos/papers/JLP2013.pdf",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        # dl_manager is a datasets.download.DownloadManager that can be used to
        # download and extract URLs
        dl_dir = dl_manager.download_and_extract(_URL)

        # This folder contains the orginal/2013 dataset
        data_dir = os.path.join(dl_dir, "Guardian", "Guardian_original")

        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"data_dir": data_dir, "samples_folders": self.config.train_folder, "split": "train"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.TEST,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"data_dir": data_dir, "samples_folders": self.config.test_folder, "split": "test"},
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={"data_dir": data_dir, "samples_folders": self.config.valid_folder, "split": "valid"},
            ),
        ]

    def _generate_examples(self, data_dir, samples_folders, split):
        """Yields examples."""
        # Yields (key, example) tuples from the dataset

        # Training and validation are on 1 topic/genre, while testing is on multiple topics
        # We convert the sample folders into list (from string)
        if samples_folders.count(",") == 0:
            samples_folders = [samples_folders]
        else:
            samples_folders = samples_folders.split(",")

        # the dataset is structured as:
        # |-Topic1
        # |---author 1
        # |------- article-1
        # |------- article-2
        # |---author 2
        # |------- article-1
        # |------- article-2
        # |-Topic2
        # ...

        for topic in samples_folders:
            full_path = os.path.join(data_dir, topic)

            for author in os.listdir(full_path):

                list_articles = os.listdir(os.path.join(full_path, author))
                if len(list_articles) == 0:
                    # Some authors have no articles on certain topics
                    continue

                for id_, article in enumerate(list_articles):
                    path_2_author = os.path.join(full_path, author)
                    path_2_article = os.path.join(path_2_author, article)

                    with open(path_2_article, "r", encoding="utf8", errors="ignore") as f:
                        art = f.readlines()

                    # The whole article is stored as one line. We access the 1st element of the list
                    # to store it as string, not as a list
                    yield f"{topic}_{author}_{id_}", {
                        "article": art[0],
                        "author": author,
                        "topic": topic,
                    }



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""TODO: Add a description here."""


import pickle

import datasets


# Find for instance the citation on arxiv or on the dataset repo/website
_CITATION = """\
@article{ladhak-wiki-2020,
  title   = {WikiLingua: A New Benchmark Dataset for Multilingual Abstractive Summarization},
  authors = {Faisal Ladhak, Esin Durmus, Claire Cardie and Kathleen McKeown},
  journal = {arXiv preprint arXiv:2010.03093},
  year    = {2020},
  url     = {https://arxiv.org/abs/2010.03093}
}
"""

_DESCRIPTION = """\
WikiLingua is a large-scale multilingual dataset for the evaluation of
crosslingual abstractive summarization systems. The dataset includes ~770k
article and summary pairs in 18 languages from WikiHow. The gold-standard
article-summary alignments across languages was done by aligning the images
that are used to describe each how-to step in an article.
"""

_HOMEPAGE = "https://github.com/esdurmus/Wikilingua"

_LICENSE = "CC BY-NC-SA 3.0"

# Download links
_URLs = {
    "arabic": "https://drive.google.com/uc?export=download&id=1__EjA6oZsgXQpggPm-h54jZu3kP6Y6zu",
    "chinese": "https://drive.google.com/uc?export=download&id=1TuWH7uwu6V90QWmZn25qhou1rm97Egmn",
    "czech": "https://drive.google.com/uc?export=download&id=1GcUN6mytEcOMBBOvjJOQzBmEkc-LdgQg",
    "dutch": "https://drive.google.com/uc?export=download&id=1-w-0uqaC6hnRn1F_3XqJEvi09zlcTIhX",
    "english": "https://drive.google.com/uc?export=download&id=11wMGqNVSwwk6zUnDaJEgm3qT71kAHeff",
    "french": "https://drive.google.com/uc?export=download&id=1Uit4Og1pk-br_0UJIO5sdhApyhTuHzqo",
    "german": "https://drive.google.com/uc?export=download&id=1meSNZHxd_0TZLKCRCYGN-Ke3IA5c1qOE",
    "hindi": "https://drive.google.com/uc?export=download&id=1ZyFGufe4puX3vjGPbp4xg9Hca3Gwq22g",
    "indonesian": "https://drive.google.com/uc?export=download&id=1PGa8j1_IqxiGTc3SU6NMB38sAzxCPS34",
    "italian": "https://drive.google.com/uc?export=download&id=1okwGJiOZmTpNRNgJLCnjFF4Q0H1z4l6_",
    "japanese": "https://drive.google.com/uc?export=download&id=1Z2ty5hU0tIGRZRDlFQZLO7b5vijRfvo0",
    "korean": "https://drive.google.com/uc?export=download&id=1cqu_YAgvlyVSzzjcUyP1Cz7q0k8Pw7vN",
    "portuguese": "https://drive.google.com/uc?export=download&id=1GTHUJxxmjLmG2lnF9dwRgIDRFZaOY3-F",
    "russian": "https://drive.google.com/uc?export=download&id=1fUR3MqJ8jTMka6owA0S-Fe6aHmiophc_",
    "spanish": "https://drive.google.com/uc?export=download&id=17FGi8KI9N9SuGe7elM8qU8_3fx4sfgTr",
    "thai": "https://drive.google.com/uc?export=download&id=1QsV8C5EPJrQl37mwva_5-IJOrCaOi2tH",
    "turkish": "https://drive.google.com/uc?export=download&id=1M1M5yIOyjKWGprc3LUeVVwxgKXxgpqxm",
    "vietnamese": "https://drive.google.com/uc?export=download&id=17FGi8KI9N9SuGe7elM8qU8_3fx4sfgTr",
}


class WikiLingua(datasets.GeneratorBasedBuilder):
    """TODO: Short description of my dataset."""

    VERSION = datasets.Version("1.1.0")

    # This is an example of a dataset with multiple configurations.
    # If you don't want/need to define several sub-sets in your dataset,
    # just remove the BUILDER_CONFIG_CLASS and the BUILDER_CONFIGS attributes.

    # If you need to make complex sub-parts in the datasets with configurable options
    # You can create your own builder configuration class to store attribute, inheriting from datasets.BuilderConfig
    # BUILDER_CONFIG_CLASS = MyBuilderConfig

    # You will  be able to load one or the other configurations in the following list with
    # data = datasets.load_dataset('my_dataset', 'first_domain')
    # data = datasets.load_dataset('my_dataset', 'second_domain')
    BUILDER_CONFIGS = [
        datasets.BuilderConfig(name="arabic", version=VERSION, description="A subset of article-summary in Arabic"),
        datasets.BuilderConfig(name="chinese", version=VERSION, description="A subset of article-summary in Chinese"),
        datasets.BuilderConfig(name="czech", version=VERSION, description="A subset of article-summary in Czech"),
        datasets.BuilderConfig(name="dutch", version=VERSION, description="A subset of article-summary in Dutch"),
        datasets.BuilderConfig(name="english", version=VERSION, description="A subset of article-summary in English"),
        datasets.BuilderConfig(name="french", version=VERSION, description="A subset of article-summary in  French"),
        datasets.BuilderConfig(name="german", version=VERSION, description="A subset of article-summary in German"),
        datasets.BuilderConfig(name="hindi", version=VERSION, description="A subset of article-summary in Hindi"),
        datasets.BuilderConfig(
            name="indonesian", version=VERSION, description="A subset of article-summary in Indonesian"
        ),
        datasets.BuilderConfig(name="italian", version=VERSION, description="A subset of article-summary in Italian"),
        datasets.BuilderConfig(
            name="japanese", version=VERSION, description="A subset of article-summary in Japanese"
        ),
        datasets.BuilderConfig(name="korean", version=VERSION, description="A subset of article-summary in Korean"),
        datasets.BuilderConfig(
            name="portuguese", version=VERSION, description="A subset of article-summary in Portuguese"
        ),
        datasets.BuilderConfig(name="russian", version=VERSION, description="A subset of article-summary in Russian"),
        datasets.BuilderConfig(name="spanish", version=VERSION, description="A subset of article-summary in Spanish"),
        datasets.BuilderConfig(name="thai", version=VERSION, description="A subset of article-summary in Thai"),
        datasets.BuilderConfig(name="turkish", version=VERSION, description="A subset of article-summary in Turkish"),
        datasets.BuilderConfig(
            name="vietnamese", version=VERSION, description="A subset of article-summary in Vietnamese"
        ),
    ]

    DEFAULT_CONFIG_NAME = "english"

    def _info(self):
        if self.config.name == "english":
            features = datasets.Features(
                {
                    "url": datasets.Value("string"),
                    "article": datasets.Sequence(
                        {
                            "section_name": datasets.Value("string"),
                            "document": datasets.Value("string"),
                            "summary": datasets.Value("string"),
                        }
                    ),
                }
            )
        else:
            features = datasets.Features(
                {
                    "url": datasets.Value("string"),
                    "article": datasets.Sequence(
                        {
                            "section_name": datasets.Value("string"),
                            "document": datasets.Value("string"),
                            "summary": datasets.Value("string"),
                            "english_url": datasets.Value("string"),
                            "english_section_name": datasets.Value("string"),
                        }
                    ),
                }
            )

        return datasets.DatasetInfo(
            # This is the description that will appear on the datasets page.
            description=_DESCRIPTION,
            # This defines the different columns of the dataset and their types
            features=features,  # Here we define them above because they are different between the two configurations
            # If there's a common (input, target) tuple from the features,
            # specify them here. They'll be used if as_supervised=True in
            # builder.as_dataset.
            supervised_keys=None,
            # Homepage of the dataset for documentation
            homepage=_HOMEPAGE,
            # License for the dataset if available
            license=_LICENSE,
            # Citation for the dataset
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        my_urls = _URLs[self.config.name]
        # See create_dummy.py to create new dummy data
        train_fname = dl_manager.download_and_extract(my_urls)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "filepath": train_fname,
                    "split": "train",
                },
            ),
        ]

    def _process_article(self, article):
        """Parse the article and convert into list of dict"""
        processed_article = []
        for key, value in article.items():
            row = {"section_name": key, "document": value["document"], "summary": value["summary"]}

            if self.config.name != "english":
                row["english_url"] = value["english_url"]
                row["english_section_name"] = value["english_section_name"]
            processed_article.append(row)

        return processed_article

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        with open(filepath, "rb") as f:
            data = pickle.load(f)
            for id_, row in enumerate(data.items()):
                yield id_, {"url": row[0], "article": self._process_article(row[1])}



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""Web of science"""


import os

import datasets


_CITATION = """\
@inproceedings{kowsari2017HDLTex,
title={HDLTex: Hierarchical Deep Learning for Text Classification},
author={Kowsari, Kamran and Brown, Donald E and Heidarysafa, Mojtaba and Jafari Meimandi, Kiana and and Gerber, Matthew S and Barnes, Laura E},
booktitle={Machine Learning and Applications (ICMLA), 2017 16th IEEE International Conference on},
year={2017},
organization={IEEE}
}
"""

_DESCRIPTION = """\
The Web Of Science (WOS) dataset is a collection of data  of published papers
available from the Web of Science. WOS has been released in three versions: WOS-46985, WOS-11967 and WOS-5736. WOS-46985 is the
full dataset. WOS-11967 and WOS-5736 are two subsets of WOS-46985.

"""

_DATA_URL = "https://data.mendeley.com/public-files/datasets/9rw3vkcfy4/files/c9ea673d-5542-44c0-ab7b-f1311f7d61df/file_downloaded"


class WebOfScienceConfig(datasets.BuilderConfig):
    """BuilderConfig for WebOfScience."""

    def __init__(self, **kwargs):
        """BuilderConfig for WebOfScience.

        Args:
        **kwargs: keyword arguments forwarded to super.
        """
        super(WebOfScienceConfig, self).__init__(version=datasets.Version("6.0.0", ""), **kwargs)


class WebOfScience(datasets.GeneratorBasedBuilder):
    """Web of Science"""

    BUILDER_CONFIGS = [
        WebOfScienceConfig(
            name="WOS5736",
            description="""Web of Science Dataset WOS-5736: This dataset contains 5,736 documents with 11 categories which include 3 parents categories.""",
        ),
        WebOfScienceConfig(
            name="WOS11967",
            description="""Web of Science Dataset WOS-11967: This dataset contains 11,967 documents with 35 categories which include 7 parents categories.""",
        ),
        WebOfScienceConfig(
            name="WOS46985",
            description="""Web of Science Dataset WOS-46985: This dataset contains 46,985 documents with 134 categories which include 7 parents categories.""",
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION + self.config.description,
            features=datasets.Features(
                {
                    "input_data": datasets.Value("string"),
                    "label": datasets.Value("int32"),
                    "label_level_1": datasets.Value("int32"),
                    "label_level_2": datasets.Value("int32"),
                }
            ),
            # No default supervised_keys (as we have to pass both premise
            # and hypothesis as input).
            supervised_keys=None,
            homepage="https://data.mendeley.com/datasets/9rw3vkcfy4/6",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""

        # dl_manager is a datasets.download.DownloadManager that can be used to

        dl_path = dl_manager.download_and_extract(_DATA_URL)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                # These kwargs will be passed to _generate_examples
                gen_kwargs={
                    "input_file": os.path.join(dl_path, self.config.name, "X.txt"),
                    "label_file": os.path.join(dl_path, self.config.name, "Y.txt"),
                    "label_level_1_file": os.path.join(dl_path, self.config.name, "YL1.txt"),
                    "label_level_2_file": os.path.join(dl_path, self.config.name, "YL2.txt"),
                },
            )
        ]

    def _generate_examples(self, input_file, label_file, label_level_1_file, label_level_2_file):
        """Yields examples."""
        with open(input_file, encoding="utf-8") as f:
            input_data = f.readlines()
        with open(label_file, encoding="utf-8") as f:
            label_data = f.readlines()
        with open(label_level_1_file, encoding="utf-8") as f:
            label_level_1_data = f.readlines()
        with open(label_level_2_file, encoding="utf-8") as f:
            label_level_2_data = f.readlines()
        for i in range(len(input_data)):
            yield i, {
                "input_data": input_data[i],
                "label": label_data[i],
                "label_level_1": label_level_1_data[i],
                "label_level_2": label_level_2_data[i],
            }



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""WMT18: Translate dataset."""

import datasets

from .wmt_utils import CWMT_SUBSET_NAMES, Wmt, WmtConfig


_URL = "http://www.statmt.org/wmt18/translation-task.html"
_CITATION = """\
@InProceedings{bojar-EtAl:2018:WMT1,
  author    = {Bojar, Ond\v{r}ej  and  Federmann, Christian  and  Fishel, Mark
    and Graham, Yvette  and  Haddow, Barry  and  Huck, Matthias  and
    Koehn, Philipp  and  Monz, Christof},
  title     = {Findings of the 2018 Conference on Machine Translation (WMT18)},
  booktitle = {Proceedings of the Third Conference on Machine Translation,
    Volume 2: Shared Task Papers},
  month     = {October},
  year      = {2018},
  address   = {Belgium, Brussels},
  publisher = {Association for Computational Linguistics},
  pages     = {272--307},
  url       = {http://www.aclweb.org/anthology/W18-6401}
}
"""

_LANGUAGE_PAIRS = [(lang, "en") for lang in ["cs", "de", "et", "fi", "kk", "ru", "tr", "zh"]]


class Wmt18(Wmt):
    """WMT 18 translation datasets for all {xx, "en"} language pairs."""

    # Version history:
    # 1.0.0: S3 (new shuffling, sharding and slicing mechanism).
    BUILDER_CONFIGS = [
        WmtConfig(  # pylint:disable=g-complex-comprehension
            description="WMT 2018 %s-%s translation task dataset." % (l1, l2),
            url=_URL,
            citation=_CITATION,
            language_pair=(l1, l2),
            version=datasets.Version("1.0.0"),
        )
        for l1, l2 in _LANGUAGE_PAIRS
    ]

    @property
    def manual_download_instructions(self):
        if self.config.language_pair[1] in ["cs", "hi", "ru"]:
            return "Please download the data manually as explained. TODO(PVP)"

    @property
    def _subsets(self):
        return {
            datasets.Split.TRAIN: [
                "europarl_v7",
                "europarl_v8_18",
                "paracrawl_v1",
                "commoncrawl",
                "newscommentary_v13",
                "czeng_17",
                "yandexcorpus",
                "wikiheadlines_fi",
                "wikiheadlines_ru",
                "setimes_2",
                "uncorpus_v1",
                "rapid_2016",
            ]
            + CWMT_SUBSET_NAMES,
            datasets.Split.VALIDATION: ["newsdev2018", "newstest2017", "newstestB2017"],
            datasets.Split.TEST: ["newstest2018"],
        }



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
import os

import datasets


_DESCRIPTION = """\
This is the Tilde MODEL Corpus – Multilingual Open Data for European Languages.

The data has been collected from sites allowing free use and reuse of its content, as well as from Public Sector web sites. The activities have been undertaken as part of the ODINE Open Data Incubator for Europe, which aims to support the next generation of digital businesses and fast-track the development of new products and services. The corpus includes the following parts:
Tilde MODEL - EESC is a multilingual corpus compiled from document texts of European Economic and Social Committee document portal. Source: http://dm.eesc.europa.eu/
Tilde MODEL - RAPID multilingual parallel corpus is compiled from all press releases of Press Release Database of European Commission released between 1975 and end of 2016 as available from http://europa.eu/rapid/
Tilde MODEL - ECB multilingual parallel corpus is compiled from the multilingual pages of European Central Bank web site http://ebc.europa.eu/
Tilde MODEL - EMA is a corpus compiled from texts of European Medicines Agency document portal as available in http://www.ema.europa.eu/ at the end of 2016
Tilde MODEL - World Bank is a corpus compiled from texts of World Bank as available in http://www.worldbank.org/ in 2017
Tilde MODEL - AirBaltic.com Travel Destinations is a multilingual parallel corpus compiled from description texts of AirBaltic.com travel destinations as available in https://www.airbaltic.com/en/destinations/ in 2017
Tilde MODEL - LiveRiga.com is a multilingual parallel corpus compiled from Riga tourist attractions description texts of http://liveriga.com/ web site in 2017
Tilde MODEL - Lithuanian National Philharmonic Society is a parallel corpus compiled from texts of Lithuanian National Philharmonic Society web site http://www.filharmonija.lt/ in 2017
Tilde MODEL - mupa.hu is a parallel corpus from texts of Müpa Budapest - web site of Hungarian national culture house and concert venue https://www.mupa.hu/en/ compiled in spring of 2017
Tilde MODEL - fold.lv is a parallel corpus from texts of fold.lv portal http://www.fold.lv/en/ of the best of Latvian and foreign creative industries as compiled in spring of 2017
Tilde MODEL - czechtourism.com is a multilingual parallel corpus from texts of http://czechtourism.com/ portal compiled in spring of 2017
30 languages, 274 bitexts
total number of files: 125
total number of tokens: 1.43G
total number of sentence fragments: 62.44M
"""
_HOMEPAGE_URL = "http://opus.nlpl.eu/TildeMODEL.php"
_CITATION = """\
Roberts Rozis, Raivis Skadins, 2017, Tilde MODEL - Multilingual Open Data for EU Languages. Proceedings of the 21th Nordic Conference of Computational Linguistics NODALIDA 2017
"""

_VERSION = "2018.0.0"
_BASE_NAME = "TildeMODEL.{}.{}"
_BASE_URL = "https://object.pouta.csc.fi/OPUS-TildeMODEL/v2018/moses/{}-{}.txt.zip"

# Please note that only few pairs are shown here. You can use config to generate data for all language pairs
_LANGUAGE_PAIRS = [
    ("bg", "el"),
    ("cs", "en"),
    ("de", "hr"),
    ("en", "no"),
    ("es", "pt"),
]


class TildeModelConfig(datasets.BuilderConfig):
    def __init__(self, *args, lang1=None, lang2=None, **kwargs):
        super().__init__(
            *args,
            name=f"{lang1}-{lang2}",
            **kwargs,
        )
        self.lang1 = lang1
        self.lang2 = lang2


class TildeModel(datasets.GeneratorBasedBuilder):
    BUILDER_CONFIGS = [
        TildeModelConfig(
            lang1=lang1,
            lang2=lang2,
            description=f"Translating {lang1} to {lang2} or vice versa",
            version=datasets.Version(_VERSION),
        )
        for lang1, lang2 in _LANGUAGE_PAIRS
    ]
    BUILDER_CONFIG_CLASS = TildeModelConfig

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("string"),
                    "translation": datasets.Translation(languages=(self.config.lang1, self.config.lang2)),
                },
            ),
            supervised_keys=None,
            homepage=_HOMEPAGE_URL,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        def _base_url(lang1, lang2):
            return _BASE_URL.format(lang1, lang2)

        download_url = _base_url(self.config.lang1, self.config.lang2)
        path = dl_manager.download_and_extract(download_url)
        return [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={"datapath": path},
            )
        ]

    def _generate_examples(self, datapath):
        l1, l2 = self.config.lang1, self.config.lang2
        folder = l1 + "-" + l2
        l1_file = _BASE_NAME.format(folder, l1)
        l2_file = _BASE_NAME.format(folder, l2)
        l1_path = os.path.join(datapath, l1_file)
        l2_path = os.path.join(datapath, l2_file)
        with open(l1_path, encoding="utf-8") as f1, open(l2_path, encoding="utf-8") as f2:
            for sentence_counter, (x, y) in enumerate(zip(f1, f2)):
                x = x.strip()
                y = y.strip()
                result = (
                    sentence_counter,
                    {
                        "id": str(sentence_counter),
                        "translation": {l1: x, l2: y},
                    },
                )
                yield result



# coding=utf-8
# Copyright 2020 The TensorFlow Datasets Authors and the HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""AG News topic classification dataset."""


import csv

import datasets
from datasets.tasks import TextClassification


_DESCRIPTION = """\
AG is a collection of more than 1 million news articles. News articles have been
gathered from more than 2000 news sources by ComeToMyHead in more than 1 year of
activity. ComeToMyHead is an academic news search engine which has been running
since July, 2004. The dataset is provided by the academic comunity for research
purposes in data mining (clustering, classification, etc), information retrieval
(ranking, search, etc), xml, data compression, data streaming, and any other
non-commercial activity. For more information, please refer to the link
http://www.di.unipi.it/~gulli/AG_corpus_of_news_articles.html .

The AG's news topic classification dataset is constructed by Xiang Zhang
(xiang.zhang@nyu.edu) from the dataset above. It is used as a text
classification benchmark in the following paper: Xiang Zhang, Junbo Zhao, Yann
LeCun. Character-level Convolutional Networks for Text Classification. Advances
in Neural Information Processing Systems 28 (NIPS 2015).
"""

_CITATION = """\
@inproceedings{Zhang2015CharacterlevelCN,
  title={Character-level Convolutional Networks for Text Classification},
  author={Xiang Zhang and Junbo Jake Zhao and Yann LeCun},
  booktitle={NIPS},
  year={2015}
}
"""

_TRAIN_DOWNLOAD_URL = "https://raw.githubusercontent.com/mhjabreel/CharCnn_Keras/master/data/ag_news_csv/train.csv"
_TEST_DOWNLOAD_URL = "https://raw.githubusercontent.com/mhjabreel/CharCnn_Keras/master/data/ag_news_csv/test.csv"


class AGNews(datasets.GeneratorBasedBuilder):
    """AG News topic classification dataset."""

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "text": datasets.Value("string"),
                    "label": datasets.features.ClassLabel(names=["World", "Sports", "Business", "Sci/Tech"]),
                }
            ),
            homepage="http://groups.di.unipi.it/~gulli/AG_corpus_of_news_articles.html",
            citation=_CITATION,
            task_templates=[TextClassification(text_column="text", label_column="label")],
        )

    def _split_generators(self, dl_manager):
        train_path = dl_manager.download_and_extract(_TRAIN_DOWNLOAD_URL)
        test_path = dl_manager.download_and_extract(_TEST_DOWNLOAD_URL)
        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": train_path}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": test_path}),
        ]

    def _generate_examples(self, filepath):
        """Generate AG News examples."""
        with open(filepath, encoding="utf-8") as csv_file:
            csv_reader = csv.reader(
                csv_file, quotechar='"', delimiter=",", quoting=csv.QUOTE_ALL, skipinitialspace=True
            )
            for id_, row in enumerate(csv_reader):
                label, title, description = row
                # Original labels are [1, 2, 3, 4] ->
                #                   ['World', 'Sports', 'Business', 'Sci/Tech']
                # Re-map to [0, 1, 2, 3].
                label = int(label) - 1
                text = " ".join((title, description))
                yield id_, {"text": text, "label": label}



# coding=utf-8
# Copyright 2020 The HuggingFace Datasets Authors and the current dataset script contributor.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""SWAG dataset."""


import csv

import datasets


_CITATION = """\
@inproceedings{zellers2018swagaf,
    title={SWAG: A Large-Scale Adversarial Dataset for Grounded Commonsense Inference},
    author={Zellers, Rowan and Bisk, Yonatan and Schwartz, Roy and Choi, Yejin},
    booktitle = "Proceedings of the 2018 Conference on Empirical Methods in Natural Language Processing (EMNLP)",
    year={2018}
}
"""

_DESCRIPTION = """\
Given a partial description like "she opened the hood of the car,"
humans can reason about the situation and anticipate what might come
next ("then, she examined the engine"). SWAG (Situations With Adversarial Generations)
is a large-scale dataset for this task of grounded commonsense
inference, unifying natural language inference and physically grounded reasoning.

The dataset consists of 113k multiple choice questions about grounded situations
(73k training, 20k validation, 20k test).
Each question is a video caption from LSMDC or ActivityNet Captions,
with four answer choices about what might happen next in the scene.
The correct answer is the (real) video caption for the next event in the video;
the three incorrect answers are adversarially generated and human verified,
so as to fool machines but not humans. SWAG aims to be a benchmark for
evaluating grounded commonsense NLI and for learning representations.

The full data contain more information,
but the regular configuration will be more interesting for modeling
(note that the regular data are shuffled). The test set for leaderboard submission
is under the regular configuration.
"""

_LICENSE = "Unknown"

_URLs = {
    "full": {
        "train": "https://raw.githubusercontent.com/rowanz/swagaf/master/data/train_full.csv",
        "val": "https://raw.githubusercontent.com/rowanz/swagaf/master/data/val_full.csv",
    },
    "regular": {
        "train": "https://raw.githubusercontent.com/rowanz/swagaf/master/data/train.csv",
        "val": "https://raw.githubusercontent.com/rowanz/swagaf/master/data/val.csv",
        "test": "https://raw.githubusercontent.com/rowanz/swagaf/master/data/test.csv",
    },
}


class Swag(datasets.GeneratorBasedBuilder):
    """SWAG dataset"""

    VERSION = datasets.Version("1.1.0")

    BUILDER_CONFIGS = [
        datasets.BuilderConfig(name="regular", description="The configuration to use for modeling."),
        datasets.BuilderConfig(name="full", description="The full data."),
    ]

    DEFAULT_CONFIG_NAME = "regular"

    def _info(self):
        if self.config.name == "regular":
            features = datasets.Features(
                {
                    "video-id": datasets.Value("string"),
                    "fold-ind": datasets.Value("string"),
                    "startphrase": datasets.Value("string"),
                    "sent1": datasets.Value("string"),
                    "sent2": datasets.Value("string"),
                    "gold-source": datasets.Value("string"),
                    "ending0": datasets.Value("string"),
                    "ending1": datasets.Value("string"),
                    "ending2": datasets.Value("string"),
                    "ending3": datasets.Value("string"),
                    "label": datasets.ClassLabel(names=["0", "1", "2", "3"]),
                }
            )
        else:
            features = datasets.Features(
                {
                    "video-id": datasets.Value("string"),
                    "fold-ind": datasets.Value("string"),
                    "startphrase": datasets.Value("string"),
                    "gold-ending": datasets.Value("string"),
                    "distractor-0": datasets.Value("string"),
                    "distractor-1": datasets.Value("string"),
                    "distractor-2": datasets.Value("string"),
                    "distractor-3": datasets.Value("string"),
                    "gold-source": datasets.Value("string"),
                    "gold-type": datasets.Value("string"),
                    "distractor-0-type": datasets.Value("string"),
                    "distractor-1-type": datasets.Value("string"),
                    "distractor-2-type": datasets.Value("string"),
                    "distractor-3-type": datasets.Value("string"),
                    "sent1": datasets.Value("string"),
                    "sent2": datasets.Value("string"),
                }
            )
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=features,
            supervised_keys=None,
            homepage="https://rowanzellers.com/swag/",
            license=_LICENSE,
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        my_urls = _URLs[self.config.name]
        data_dir = dl_manager.download_and_extract(my_urls)

        splits = [
            datasets.SplitGenerator(
                name=datasets.Split.TRAIN,
                gen_kwargs={
                    "filepath": data_dir["train"],
                    "split": "train",
                },
            ),
            datasets.SplitGenerator(
                name=datasets.Split.VALIDATION,
                gen_kwargs={
                    "filepath": data_dir["val"],
                    "split": "val",
                },
            ),
        ]
        if self.config.name == "regular":
            splits.append(
                datasets.SplitGenerator(
                    name=datasets.Split.TEST,
                    gen_kwargs={"filepath": data_dir["test"], "split": "test"},
                )
            )

        return splits

    def _generate_examples(self, filepath, split):
        """Yields examples."""
        with open(filepath, "r", encoding="utf-8") as f:
            lines = list(csv.reader(f, delimiter=","))

            for id_, row in enumerate(lines[1:]):
                if self.config.name == "regular":
                    yield id_, {
                        "video-id": row[1],
                        "fold-ind": row[2],
                        "startphrase": row[3],
                        "sent1": row[4],
                        "sent2": row[5],
                        "gold-source": row[6],
                        "ending0": row[7],
                        "ending1": row[8],
                        "ending2": row[9],
                        "ending3": row[10],
                        "label": -1 if split == "test" else row[11],
                    }
                else:
                    yield id_, {
                        "video-id": row[0],
                        "fold-ind": row[1],
                        "startphrase": row[2],
                        "gold-ending": row[3],
                        "distractor-0": row[4],
                        "distractor-1": row[5],
                        "distractor-2": row[6],
                        "distractor-3": row[7],
                        "gold-source": row[8],
                        "gold-type": row[9],
                        "distractor-0-type": row[10],
                        "distractor-1-type": row[11],
                        "distractor-2-type": row[12],
                        "distractor-3-type": row[13],
                        "sent1": row[14],
                        "sent2": row[15],
                    }



# coding=utf-8
# Copyright 2020 HuggingFace Datasets Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Lint as: python3
"""DREAM: A Challenge Dataset and Models for Dialogue-Based Reading Comprehension"""


import json

import datasets


logger = datasets.logging.get_logger(__name__)


_CITATION = """\
@article{sundream2018,
  title={{DREAM}: A Challenge Dataset and Models for Dialogue-Based Reading Comprehension},
  author={Sun, Kai and Yu, Dian and Chen, Jianshu and Yu, Dong and Choi, Yejin and Cardie, Claire},
  journal={Transactions of the Association for Computational Linguistics},
  year={2019},
  url={https://arxiv.org/abs/1902.00164v1}
}
"""

_DESCRIPTION = """\
DREAM is a multiple-choice Dialogue-based REAding comprehension exaMination dataset. \
In contrast to existing reading comprehension datasets, DREAM is the first to focus on \
in-depth multi-turn multi-party dialogue understanding.
"""

_URL = "https://raw.githubusercontent.com/nlpdata/dream/master/data/"
_URLS = {
    "train": _URL + "train.json",
    "dev": _URL + "dev.json",
    "test": _URL + "test.json",
}


class DreamConfig(datasets.BuilderConfig):
    """BuilderConfig for Dream."""

    def __init__(self, **kwargs):
        """BuilderConfig for Dream.

        Args:
          **kwargs: keyword arguments forwarded to super.
        """
        super(DreamConfig, self).__init__(**kwargs)


class Dream(datasets.GeneratorBasedBuilder):
    """DREAM: A Challenge Dataset and Models for Dialogue-Based Reading Comprehension"""

    VERSION = datasets.Version("1.0.0")

    BUILDER_CONFIGS = [
        DreamConfig(
            name="plain_text",
            version=datasets.Version("1.0.0"),
            description="plain_text",
        ),
    ]

    def _info(self):
        return datasets.DatasetInfo(
            description=_DESCRIPTION,
            features=datasets.Features(
                {
                    "id": datasets.Value("int32"),
                    "dialogue_id": datasets.Value("string"),
                    "dialogue": datasets.Sequence(datasets.Value("string")),
                    "question": datasets.Value("string"),
                    "choice": datasets.features.Sequence(datasets.Value("string")),
                    "answer": datasets.Value("string"),
                }
            ),
            supervised_keys=None,
            homepage="https://dataset.org/dream/",
            citation=_CITATION,
        )

    def _split_generators(self, dl_manager):
        """Returns SplitGenerators."""
        downloaded_files = dl_manager.download_and_extract(_URLS)

        return [
            datasets.SplitGenerator(name=datasets.Split.TRAIN, gen_kwargs={"filepath": downloaded_files["train"]}),
            datasets.SplitGenerator(name=datasets.Split.VALIDATION, gen_kwargs={"filepath": downloaded_files["dev"]}),
            datasets.SplitGenerator(name=datasets.Split.TEST, gen_kwargs={"filepath": downloaded_files["test"]}),
        ]

    def _generate_examples(self, filepath):
        """This function returns the examples in the raw (text) form."""
        logger.info("⏳ Generating examples from = %s", filepath)
        with open(filepath, encoding="utf-8") as f:
            dialogues = json.load(f)
            counter = 0
            for dialogue in dialogues:
                dialogue_text = dialogue[0]
                questions = dialogue[1]
                dialogue_id = dialogue[2]

                for que in questions:
                    yield counter, {
                        "id": counter,
                        "dialogue_id": dialogue_id,
                        "dialogue": dialogue_text,
                        "question": que["question"],
                        "choice": que["choice"],
                        "answer": que["answer"],
                    }
                    counter += 1
